@@ -531,8 +531,7 @@ public:
 	 * Application operator. Apply the validation algorithm
 	 * specific to multicast addresses
 	 */
-	void 
-	operator()(const in6_addr address) const; 
+	void operator()(const in6_addr address) const; 
 };
 
 /**
