@@ -40,7 +40,7 @@
 
 /**
  * @file file.h
- * @short Files, fifos, directories, pipes and dynamic loader services.
+ * @short Files and dynamic loader services.
  **/
 
 #ifndef	CCXX_FILE_H_
@@ -76,6 +76,10 @@
 #include <direct.h>
 #endif
 
+#ifdef	HAVE_SHL_LOAD
+#include <dl.h>
+#endif
+
 #ifdef  HAVE_MACH_DYLD
 #include <mach-o/dyld.h>
 #endif
@@ -150,6 +154,7 @@ protected:
 		caddr_t address;
 		ccxx_size_t len;
 		off_t pos;
+		bool locked;
 	} fcb_t;
 
 public:
@@ -248,76 +253,6 @@ public:
 	static char *getRealpath(const char *path, char *buffer, size_t size = PATH_MAX);
 };
 
-#ifndef WIN32
-/**
- * This class provides a few alterations to the standard fstream class
- * for dealing with fifo devices.  In particular, a fifo is assumed to
- * be created via mkfifo and is destroyed when closed.  The fifo is a
- * r/w streamable object.  fifo streams are presumed unique to posix
- * systems and are generally not portable classes.
- *
- * @todo implement in win32
- */
-
-class fifostream : public std::fstream, public File
-{
-private:
-	char *pathname;
-
-public:
-	/**
-	 * Create fifo "object", but do not create or open a fifo
-	 * stream.
-	 */
-	fifostream();
-
-	/**
-	 * Create and open a fifo stream object.
-	 *
-	 * @param fname name of fifo.
-	 * @param access access flags.
-	 */
-	fifostream(const char *fname, long access = (long)attrGroup);
-
-	/**
-	 * close any open fifo and remove.
-	 */
-	virtual ~fifostream();
-
-	/**
-	 * open a new fifo stream.  Kill the currently open one if
-	 * present.
-	 *
-	 * @param fname name of fifo.
-	 * @param access access flags.
-	 */
-	void open(const char *fname, long access = (long)attrGroup);
-
-	/**
-	 * close the fifo stream and remove it.
-	 */
-	void close(void);
-};
-#endif // !WIN32
-
-#ifndef WIN32
-/**
- * The FIFOSession produces a seperate thread which can manage a fifo
- * stream session.  This is somewhat similar to TTYSession and TCPSession
- * in purpose.
- * @todo implement in win32
- */
-class FIFOSession : public Thread, public std::fstream, public File
-{
-private:
-	char *pathname;
-
-public:
-	FIFOSession(const char *session, long access = (long)attrGroup, int pri = 0, int stack = 0);
-	virtual ~FIFOSession();
-};
-#endif // !WIN32
-
 /**
  * A low level portable directory class.  Used to support ccstd Directory
  * container.  This provides a basic mechanism for allocating and
@@ -377,6 +312,83 @@ public:
 	bool isValid(void);
 };
 
+/**
+ * A generic class to walk a hierarchical directory structure.  
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short Directory tree walking.
+ */
+class __EXPORT	DirTree 
+{
+private:
+	char path[PATH_MAX + 1];
+	Dir *dir;
+	unsigned max, current, prefixpos;
+
+protected:
+	/**
+	 * Virtual method to filter results.  Virtual override methods
+	 * should call baseclass method to assure . and .. names are
+	 * stripped out.
+	 *
+	 * @return true if current filename is accepted.
+	 * @param file path to examine
+	 * @param stat info of type, date, etc.
+	 */
+	virtual bool filter(const char *path, struct stat *ino);
+
+public:
+	/**
+	 * Construct a directory tree walk starting at the specified
+	 * prefix.  A maximum subdirectory depth is also specified.
+	 *
+	 * @param prefix to start walk.
+	 * @param maximum subdirectory.
+	 */
+	DirTree(const char *prefix, unsigned depth);
+
+	/**
+	 * Construct an un-opened directory tree of a known maximum depth
+	 *
+	 * @param maximum subdirectory.
+ 	 */
+	DirTree(unsigned depth);
+
+	virtual ~DirTree();
+
+	/**
+	 * Open a directory tree path.
+	 *
+	 * @param path to open.
+	 */
+	void open(const char *prefix);
+
+	/**
+	 * Close the directory path.
+	 */
+	void close(void);
+
+	/**
+	 * Extract the next full pathname from the directory walk.
+	 * When returning directories, a '/' is appended.  The
+	 * returned string is a buffer of MAX_PATH size.
+	 *
+	 * @return path of next subdirectory entry or NULL.
+	 */
+	char *getPath(void);
+
+	/**
+	 * This is used to step through the filter virtual for an
+	 * entire subtree, and is used for cases where a derived
+	 * DirTree class performs it's primary operations through
+	 * filter rather than externally by calling getPath().
+	 *
+	 * @return number of files and directories examined.
+	 * @param path to examine.
+	 */
+	unsigned perform(const char *prefix);
+};
+
 /**
  * The purpose of this class is to define a base class for low level
  * random file access that is portable between Win32 and Posix systems.
@@ -764,6 +776,16 @@ public:
 	 */
 	MappedFile(const char *fname, Access mode);
 
+	/**
+	 * Create if not exists, and map a file of specified size
+	 * into memory.
+	 *
+	 * @param fname file name to access for mapping.
+	 * @param mode access mode to map file.
+	 * @param size of file to map.
+	 */
+	MappedFile(const char *fname, Access mode, size_t size);
+
 	/**
 	 * Map a portion or all of a specified file in the specified
 	 * shared memory access mode.  Valid mapping modes include
@@ -845,6 +867,26 @@ public:
 	 * @param len size of memory segment to map.
 	 */
 	caddr_t fetch(off_t pos, size_t len);
+
+	/**
+	 * Lock the currently mapped portion of a file.
+	 *
+	 * @return true if pages are locked.
+	 */
+	bool lock(void);
+
+	/**
+	 * Unlock a locked mapped portion of a file.
+	 */
+	void unlock(void);
+
+	/**
+	 * Compute map size to aligned page boundry.
+	 *
+	 * @param size request.
+	 * @return page aligned size.
+	 */
+	size_t pageAligned(size_t size);
 };
 
 
@@ -959,6 +1001,10 @@ inline bool isDevice(const char *path)
 bool __EXPORT canAccess(const char *path);
 /** @relates RandomFile */
 bool __EXPORT canModify(const char *path);
+/** @relates RandomFile */
+time_t __EXPORT lastModified(const char *path);
+/** @relates RandomFile */
+time_t __EXPORT lastAccessed(const char *path);
 
 #ifdef	COMMON_STD_EXCEPTION
 
@@ -974,18 +1020,6 @@ public:
 	DSOException(const String &str) : IOException(str) {};
 };
 
-class __EXPORT FIFOException : public IOException
-{
-public:
-	FIFOException(const String &str) : IOException(str) {};
-};
-
-class __EXPORT PipeException : public IOException
-{
-public:
-	PipeException(const String &str) : IOException(str) {};
-};
-
 class __EXPORT FileException : public IOException
 {
 public:
