@@ -91,6 +91,7 @@ class __EXPORT MemPager
 {
 private:
 	friend class String;
+	friend class MemPagerObject;
 
 	size_t pagesize;
 	unsigned int pages;
@@ -176,6 +177,64 @@ public:
 		{return pages;};
 };
 
+/**
+ * The StackPager provides a repository to stash and retrieve working
+ * data in last-in-first-out order.  The use of a mempager to support
+ * it's operation allows storage of arbitrary sized objects with no
+ * fixed limit.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short last in first out object pager.
+ */
+class __EXPORT StackPager : protected MemPager
+{
+private:
+	typedef struct frame
+	{
+		struct frame *next;
+		char data[1];
+	}	frame_t;
+
+	frame_t *stack;
+
+public:
+	/**
+	 * Create a lifo pager as a mempager.
+	 *
+	 * @param page size for allocation
+	 */
+	StackPager(size_t pagesize);
+
+	/**
+	 * Push an arbitrary object onto the stack.
+	 *
+	 * @return stack memory location.
+	 * @param pointer to data
+	 * @param size of data.
+	 */
+	void *push(const void *object, size_t len);
+
+	/**
+	 * Push a string onto the stack.
+	 *
+	 * @return stack memory location.
+	 * @param string pointer.
+	 */
+	void *push(const char *string);
+
+	/**
+	 * Retrieve next object from stack.
+	 *
+	 * @return object.
+	 */
+	void *pull(void);
+
+	/**
+	 * Purge the stack of all objects and memory allocations.
+	 */
+	void purge(void);
+};
+
 /**
  * The shared mempager uses a mutex to protect key access methods.
  * This class is used when a mempager will be shared by multiple
@@ -337,6 +396,7 @@ private:
 protected:
 	Keysym* getSymbol(const char *sym, bool create);
 
+public:
 	/**
 	 * Load additional key values into the currrent object from
 	 * the specfied config source (a config file/section pair).
@@ -370,6 +430,17 @@ protected:
 	void loadPrefix(const char *prefix,
 		  const char *keypath, const char *environment = "CONFIG_KEYDATA");
 
+	/**
+	 * Load additional keys into the current object using a real
+	 * filename that is directly passed rather than a computed key
+	 * path.  This also uses a [keys] section as passed to the object.
+	 *
+	 * @param filename to load from
+	 * @param key value set to parse, or NULL to parse from head
+	 * @param optional key prefix
+	 */
+	void loadFile(const char *filepath, const char *keys = NULL, const char *pre = NULL);
+
 	/**
 	 * Load default keywords into the current object.  This only
 	 * loads keyword entries which have not already been defined
@@ -380,7 +451,6 @@ protected:
 	 */
 	void load(Define *pairs);
 	
-public:
 	/**
 	 * Create an empty key data object.
 	 */
@@ -506,251 +576,42 @@ public:
 };
 
 /**
- * Splits delimited string into tokens.
+ * This class is used to create derived classes which are constructed
+ * within a memory pager pool.  
  *
- * The StringTokenizer takes a pointer to a string and a pointer
- * to a string containing a number of possible delimiters.
- * The StringTokenizer provides an input forward iterator which allows
- * to iterate through all tokens. An iterator behaves like a logical
- * pointer to the tokens, i.e. to shift to the next token, you've 
- * to increment the iterator, you get the token by dereferencing the
- * iterator.
- *
- * Memory consumption:
- * This class operates on the original string and only allocates memory
- * for the individual tokens actually requested, so this class 
- * allocates at maximum the space required for the longest token in the 
- * given string.
- * Since for each iteration, memory is reclaimed for the last token,
- * you MAY NOT store pointers to them; if you need them afterwards,
- * copy them. You may not modify the original string while you operate
- * on it with the StringTokenizer; the behaviour is undefined in that
- * case.
- *
- * The iterator has one special method 'nextDelimiter()' which returns
- * a character containing the next delimiter following this
- * tokenization process or '\\0', if there are no following delimiters. In
- * case of skipAllDelim, it returns the FIRST delimiter.
- *
- * With the method 'setDelimiters(const char*)' you may change the
- * set of delimiters. It affects all running iterators.
- *
- * Example:
- * <code><pre>
- *  StringTokenizer st("mary had a little lamb;its fleece was..", " ;");
- *  StringTokenizer::iterator i;
- *  for (i = st.begin() ; i != st.end() ; ++i) {
- *        cout << "Token: '" << *i << "'\t";
- *        cout << " next Delim: '" << i.nextDelimiter() << "'" << endl;
- *  }
- *  </pre></code>
- *
- * @author Henner Zeller <H.Zeller@acm.org>
- * @license LGPL
+ * @short create objects in a memory pager.
+ * @author David Sugar <dyfet@ostel.com>
  */
-class __EXPORT StringTokenizer {
+class __EXPORT MemPagerObject
+{
 public:
 	/**
-	 * a delimiter string containing all usual whitespace delimiters.
-	 * These are space, tab, newline, carriage return,
-	 * formfeed and vertical tab. (see isspace() manpage).
-	 */
-	static const char * const SPACE;
-
-	/**
-	 * Exception thrown, if someone tried to read beyond the
-	 * end of the tokens.
-	 * Will not happen if you use it the 'clean' way with comparison
-	 * against end(), but if you skip some tokens, because you 'know'
-	 * they are there. Simplifies error handling a lot, since you can
-	 * just read your tokens the way you expect it, and if there is some
-	 * error in the input this Exception will be thrown.
-	 */
-	// maybe move more global ?
-	class NoSuchElementException { };
-
-	/**
-	 * The input forward iterator for tokens.
-	 * @author Henner Zeller
-	 */
-	class __EXPORT iterator {
-		friend class StringTokenizer;  // access our private constructors
-	private:
-		const StringTokenizer *myTok; // my StringTokenizer
-		const char *start;      // start of current token
-		const char *tokEnd;     // end of current token (->nxDelimiter)
-		const char *endp;       // one before next token
-		char *token;            // allocated token, if requested
-
-		// for initialization of the itEnd iterator
-		iterator(const StringTokenizer &tok, const char *end) 
-			: myTok(&tok),tokEnd(0),endp(end),token(0) {}
-
-		iterator(const StringTokenizer &tok)
-			: myTok(&tok),tokEnd(0),endp(myTok->str-1),token(0) {
-			++(*this); // init first token.
-		}
-
-	public:
-		iterator() : myTok(0),start(0),tokEnd(0),endp(0),token(0) {}
-
-		// see also: comment in implementation of operator++
-		virtual ~iterator() { if (token) *token='\0'; delete [] token; }
-		
-		/**
-		 * copy constructor.
-		 */
-		// everything, but not responsible for the allocated token.
-		iterator(const iterator& i) :
-			myTok(i.myTok),start(i.start),tokEnd(i.tokEnd),
-			endp(i.endp),token(0) {}
-
-		/**
-		 * assignment operator.
-		 */
-		// everything, but not responsible for the allocated token.
-		iterator &operator = (const iterator &i) {
-			myTok = i.myTok; 
-			start = i.start; endp = i.endp; tokEnd = i.tokEnd;
-			if ( token )
-				delete [] token;
-			token = 0;
-			return *this;
-		}
-
-		/**
-		 * shifts this iterator to the next token in the string.
-		 */
-		iterator &operator ++ () THROWS (NoSuchElementException);
-
-		/**
-		 * returns the immutable string this iterator
-		 * points to or '0' if no token is available (i.e.
-		 * i == end()).
-		 * Do not store pointers to this token, since it is
-		 * invalidated for each iteration. If you need the token,
-		 * copy it (e.g. with strdup());
-		 */
-		const char*  operator *  () THROWS (NoSuchElementException);
-		
-		/**
-		 * returns the next delimiter after the current token or
-		 * '\\0', if there are no following delimiters.
-		 * It returns the very next delimiter (even if 
-		 * skipAllDelim=true).
-		 */
-		inline char nextDelimiter() const {
-			return (tokEnd) ? *tokEnd : '\0';
-		}
-		
-		/**
-		 * compares to other iterator. Usually used to
-		 * compare against the end() iterator.
-		 */
-		// only compare the end-position. speed.
-		inline bool operator == (const iterator &other) const { 
-			return (endp == other.endp);
-		}
-
-		/**
-		 * compares to other iterator. Usually used to
-		 * compare against the end() iterator.
-		 */
-		// only compare the end position. speed.
-		inline bool operator != (const iterator &other) const { 
-			return (endp != other.endp);
-		}
-	};
-private:
-	friend class StringTokenizer::iterator;
-	const char *str;
-	const char *delim;
-	bool skipAll, trim;
-	iterator itEnd;
-
-public:
-	/**
-	 * creates a new StringTokenizer for a string
-	 * and a given set of delimiters.
+	 * Allocate memory from a memory pager.
 	 *
-	 * @param  str          String to be split up. This string will
-	 *                      not be modified by this StringTokenizer,
-	 *                      but you may as well not modfiy this string
-	 *                      while tokenizing is in process, which may
-	 *                      lead to undefined behaviour.
-	 *
-	 * @param  delim        String containing the characters
-	 *                      which should be regarded as delimiters.
-	 *
-	 * @param  skipAllDelim OPTIONAL. 
-	 *                      true, if subsequent
-	 *                      delimiters should be skipped at once
-	 *                      or false, if empty tokens should
-	 *                      be returned for two delimiters with
-	 *                      no other text inbetween. The first
-	 *                      behaviour may be desirable for whitespace
-	 *                      skipping, the second for input with
-	 *                      delimited entry e.g. /etc/passwd like files
-	 *                      or CSV input.
-	 *                      NOTE, that 'true' here resembles the 
-	 *                      ANSI-C strtok(char *s,char *d) behaviour.
-	 *                      DEFAULT = false
-	 *
-	 * @param trim          OPTIONAL. 
-	 *                      true, if the tokens returned
-	 *                      should be trimmed, so that they don't have
-	 *                      any whitespaces at the beginning or end.
-	 *                      Whitespaces are any of the characters 
-	 *                      defined in StringTokenizer::SPACE.
-	 *                      If delim itself is StringTokenizer::SPACE,
-	 *                      this will result in a behaviour with 
-	 *                      skipAllDelim = true.
-	 *                      DEFAULT = false
+	 * @param size of new passed from operator.
+	 * @param pager to allocate from.
 	 */
-	StringTokenizer (const char *str,
-			 const char *delim,
-			 bool skipAllDelim = false,
-			 bool trim = false);
-	
-	/**
-	 * create a new StringTokenizer which splits the input
-	 * string at whitespaces. The tokens are stripped from
-	 * whitespaces. This means, if you change the set of
-	 * delimiters in either the 'begin(const char *delim)' method
-	 * or in 'setDelimiters()', you then get whitespace
-	 * trimmed tokens, delimited by the new set.
-	 * Behaves like StringTokenizer(s, StringTokenizer::SPACE,false,true);
-	 */
-	StringTokenizer (const char *s);
+	inline void *operator new(size_t size, MemPager &pager)
+		{return pager.alloc(size);};
 
 	/**
-	 * returns the begin iterator
+	 * Allocate array from a memory pager.
+	 *
+	 * @param size of new passed from operator.
+	 * @param pager to allocate from.
 	 */
-	iterator begin() const { 
-		return iterator(*this);
-	}
-	
-	/**
-	 * changes the set of delimiters used in subsequent
-	 * iterations.
-	 */
-	void setDelimiters (const char *d) {
-		delim = d;
-	}
-	
-	/**
-	 * returns a begin iterator with an alternate set of 
-	 * delimiters.
-	 */
-	iterator begin(const char *d) { 
-		delim = d;
-		return iterator(*this);
-	}
+	inline void *operator new[](size_t size, MemPager &pager)
+		{return pager.alloc(size);};
 
 	/**
-	 * the iterator marking the end.
+	 * Mempager delete does nothing; the pool purges.
 	 */
-	const iterator& end() const { return itEnd; }
+	inline void operator delete(void *) {};
+
+	/**
+	 * Array mempager delete does nothing; the pool purges.
+	 */
+	inline void operator delete[](void *) {};
 };
 
 #ifdef	CCXX_NAMESPACES
