@@ -107,6 +107,21 @@ public:
 	 */
 	static Trap setInterruptSignal(int signo, Trap handler);
 #endif
+	/**
+	 * Lock a process in memory.  Ideally you should be deep enough
+ 	 * where additional memallocs for functions will not kill you,
+	 * or use false for future.
+	 *
+	 * @return true if successful.
+	 * @param future pages as well...
+	 */
+	bool lock(bool future = true);
+
+	/**
+	 * Unlock process pages.
+	 */
+	void unlock(void);
+	
 	/**
 	 * Spawn a process and wait for it's exit code.  In win32
 	 * this is done with the spawn system call.  In posix,
@@ -196,6 +211,94 @@ public:
 	 * @return true if successful.
 	 */
 	static bool setGroup(const char *id);
+
+	/**
+	 * Return the effective operating system page size.
+	 *
+	 * @return system page size.
+	 */
+	static size_t getPageSize(void);
+
+	/**
+	 * Used to set process priority and optionally enable realtime.
+	 */
+	static void setPriority(int pri);
+
+	/**
+	 * Used to set process scheduling policy.
+	 */
+	static void setScheduler(const char *policy);
+
+	/**
+	 * Portable shortcut for setting realtime...
+	 */
+	static void setRealtime(int pri = 0);
+
+	/**
+	 * Return true if scheduler settable.
+	 */
+	static bool isScheduler(void);
+};
+
+/**
+ * This class is used to create a "named" lock entity that can be used
+ * to control access to a resource between multiple processes.  The
+ * posix implimentation uses a pidfile and the win32 version uses a
+ * globally visible mutex.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short System-wide named lock
+ */
+class __EXPORT Lockfile
+{
+private:
+#ifdef	WIN32
+	HANDLE	_mutex;
+	bool	_flagged;
+#else
+	char *_path;
+#endif
+
+public:
+	/**
+	 * Create a lock under a known name.
+	 *
+	 * @param system-wide name of lock
+	 */
+	Lockfile(const char *name);
+
+	/**
+	 * Create a new lock object that can be used to make locks.
+	 */
+	Lockfile();
+
+	/**
+	 * Destroy the current lock and release it.
+	 */
+	~Lockfile()
+		{unlock();};
+
+	/**
+	 * Lock a system-wide name for this process.  If the lock
+	 * is successful, return true.  If an existing lock was
+	 * already acquired, release it first.
+	 *
+	 * @return true if lock successful.
+	 * @param system-wide name to use.
+	 */
+	bool lock(const char *name);
+
+	/**
+	 * Release an acquired lock.
+	 */
+	void unlock(void);
+
+	/**
+	 * Flag if the current process has aqcuired a lock.
+	 *
+	 * @return true if we have the lock.
+	 */
+	bool isLocked(void);
 };
 
 #ifdef	CCXX_NAMESPACES
