@@ -46,15 +46,17 @@
 #ifndef	CCXX_SERIAL_H_
 #define	CCXX_SERIAL_H_
 
-#ifndef CCXX_CONFIG_H_
-#include <cc++/config.h>
+#ifndef CCXX_MISSING_H_
+#include <cc++/missing.h>
 #endif
 
 #ifndef	CCXX_THREAD_H_
 #include <cc++/thread.h>
 #endif
 
-#include <iostream>
+#ifndef	CCXX_EXCEPTION_H_
+#include <cc++/exception.h>
+#endif
 
 #ifndef	WIN32
 typedef	int HANDLE;
@@ -65,10 +67,6 @@ typedef	int HANDLE;
 namespace ost {
 #endif
 
-
-class CCXX_CLASS_EXPORT Serial;
-class CCXX_CLASS_EXPORT TTYStream;
-class CCXX_CLASS_EXPORT ttystream;
 /**
  * The Serial class is used as the base for all serial I/O services
  * under APE.  A serial is a system serial port that is used either
@@ -99,9 +97,7 @@ class CCXX_CLASS_EXPORT ttystream;
  * @author David Sugar <dyfet@ostel.com>
  * @short base class for all serial I/O services.
  */
-
-
-class CCXX_CLASS_EXPORT Serial
+class __EXPORT Serial
 {
 public:
 	enum Error
@@ -217,8 +213,8 @@ protected:
 	 *
 	 * @param errstr string or message to pass.
 	 */
-	inline void error(char *errstr)
-		{error(errExtended, errstr);};
+	inline void error(char *err)
+		{error(errExtended, err);};
 
 
 	/**
@@ -414,7 +410,7 @@ public:
 /**
  * TTY streams are used to represent serial connections that are fully
  * "streamable" objects using C++ stream classes and friends.
- *
+ * 
  * The first application relevant serial I/O class is the TTYStream class.
  * TTYStream offers a linearly buffered "streaming" I/O session with the
  * serial device.  Furthermore, traditional C++ "stream" operators (<< and
@@ -423,20 +419,16 @@ public:
  * in which one can pass initial serial device parameters immediately
  * following the device name in a single string, as in
  * "/dev/tty3a:9600,7,e,1", as an example.
- *
+ * 
  * The TTYSession aggragates a TTYStream and a Common C++ Thread which is
  * assumed to be the execution context that will be used to perform actual
  * I/O operations.  This class is very anagolous to TCPSession.
- *
+ * 
  *
  * @author David Sugar <dyfet@ostel.com>
  * @short streamable tty serial I/O class.
  */
-
-#if defined(__KCC)
-#define iostream iostream_withassign
-#endif
-class CCXX_CLASS_EXPORT TTYStream : protected std::streambuf, public Serial, public std::iostream
+class __EXPORT TTYStream : protected std::streambuf, public Serial, public std::iostream
 {
 private:
 	int doallocate();
@@ -555,7 +547,7 @@ public:
  * @short C++ "fstream" style ttystream class.
  */
 
-class CCXX_CLASS_EXPORT ttystream : public TTYStream
+class __EXPORT ttystream : public TTYStream
 {
 public:
 	/**
@@ -601,7 +593,7 @@ public:
  * @short This class is very anagolous to TCPSession.
  */
 
-class CCXX_CLASS_EXPORT TTYSession : public TTYStream, public Thread
+class TTYSession : public TTYStream, public Thread
 {
 public:
 	/**
@@ -618,9 +610,8 @@ public:
 
 //	Not support this right now.......
 //
-
-class SerialPort;
-class SerialService;
+class __EXPORT SerialPort;
+class __EXPORT SerialService;
 
 /**
  * The serial port is an internal class which is attached to and then
@@ -643,12 +634,12 @@ class SerialService;
  * @author David Sugar <dyfet@ostel.com>
  * @short base class for thread pool serviced serial I/O.
  */
-class SerialPort: public Serial, public TimerPort
+class __EXPORT SerialPort: public Serial, public TimerPort
 {
 private:
 	SerialPort *next, *prev;
 	SerialService *service;
-#ifdef	CCXX_USE_POLL
+#ifdef	USE_POLL
 	struct pollfd *ufd;
 #endif
 	bool detect_pending;
@@ -786,7 +777,7 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short Thread pool service for serial ports.
  */
-class SerialService : public Thread, private Mutex
+class __EXPORT SerialService : public Thread, private Mutex
 {
 private:
 	fd_set connect;
@@ -859,8 +850,10 @@ public:
 	 * thread begins execution with the first attached port.
 	 *
 	 * @param pri of this thread to run under.
+	 * @param stack stack size.
+	 * @param id stack ID.
 	 */
-	SerialService(int pri = 0);
+	SerialService(int pri = 0, size_t stack = 0, const char *id = NULL);
 
 	/**
 	 * Terminate the service thread and update attached objects.
@@ -882,15 +875,15 @@ public:
 
 
 #ifdef	COMMON_STD_EXCEPTION
-class SerException : public IOException
+class __EXPORT SerException : public IOException
 {
 public:
-	SerException(std::string str) : IOException(str) {};
+	SerException(const String &str) : IOException(str) {};
 };
 #endif
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif
