@@ -52,8 +52,8 @@
 
 #if defined(WIN32) && !defined(__CYGWIN32__)
 #include <io.h>
-#define	IOLEN	(unsigned)
-#define	IORET	(ssize_t)
+#define	_IOLEN64	(unsigned)
+#define	_IORET64	(ssize_t)
 #define TIMEOUT_INF ~((timeout_t) 0)
 typedef int socklen_t;
 #else
@@ -61,12 +61,12 @@ typedef int socklen_t;
 typedef int SOCKET;
 #endif
 
-#ifndef	IOLEN
-#define	IOLEN
+#ifndef	_IOLEN64
+#define	_IOLEN64
 #endif
 
-#ifndef	IORET
-#define	IORET
+#ifndef	_IORET64
+#define	_IORET64
 #endif
 
 #ifndef	MSG_DONTWAIT
@@ -82,9 +82,6 @@ namespace ost {
  */
 typedef unsigned short tpport_t;
 
-class __EXPORT SocketPort;
-class __EXPORT SocketService;
-
 /**
  * The Socket is used as the base for all Internet protocol services
  * under Common C++.  A socket is a system resource (or winsock descriptor)
@@ -144,7 +141,8 @@ public:
 		errNoDelay,
 		errExtended,
 		errLookupFail,
-		errSearchErr
+		errSearchErr,
+		errInvalidValue
 	};
 
 	typedef enum Error Error;
@@ -189,6 +187,8 @@ private:
 	friend SOCKET dupSocket(SOCKET s,Socket::State state);
 
 protected:
+	static Mutex mutex;
+
 	mutable struct
 	{
 		bool thrown: 1;
@@ -252,6 +252,53 @@ protected:
 	 */
 	Error connectError(void);
 
+	/**
+	 * Set the send limit.
+	 */
+	Error sendLimit(int limit = 2048);
+
+	/**
+	 * Set thr receive limit.
+	 */
+	Error receiveLimit(int limit = 1);
+
+	/**
+	 * Set the send timeout.
+	 *
+	 * @return errSuccess if set.
+	 * @param timeout value in millisec.
+	 */
+	Error sendTimeout(timeout_t to);
+
+	/**
+	 * Receive timeout.
+	 */
+	Error receiveTimeout(timeout_t to);
+
+	/**
+	 * Set the send buffer size.
+	 *
+	 * @return errSuccess on success, or error.
+	 * @param size of buffer.
+	 */
+	Error sendBuffer(unsigned bufsize);
+
+        /**
+         * Set the receive buffer size.
+         * 
+         * @return errSuccess on success, or error. 
+         * @param size of buffer.
+         */
+        Error receiveBuffer(unsigned bufsize);
+
+	/**
+	 * Set the total buffer size bidirectionally.
+	 *
+	 * @return errSuccess on success
+	 * @param size of buffer.
+	 */
+	Error bufferSize(unsigned bufsize);
+
 	/**
 	 * Set the subnet broadcast flag for the socket.  This enables
 	 * sending to a subnet and may require special image privileges
@@ -353,6 +400,11 @@ protected:
 	 */
 	Socket(SOCKET fd);
 
+	/**
+	 * Create an inactive socket object for base constructors.
+	 */
+	Socket();
+
 	/**
          * A socket can also be constructed from an already existing
          * Socket object. On POSIX systems, the socket file descriptor
@@ -407,6 +459,14 @@ public:
 	 */
 	virtual ~Socket();
 
+	/**
+	 * See if a specific protocol family is available in the
+	 * current runtime environment.
+	 *
+	 * @return true if family available.
+	 */
+	static bool check(Family fam);
+
 	/**
 	 * Sockets may also be duplicated by the assignment operator.
 	 */
@@ -672,6 +732,11 @@ public:
 	 */
 	UDPSocket(Family family = IPV4);
 
+	/**
+	 * Create a UDP socket bound by a service name.
+	 */
+	UDPSocket(const char *name, Family family = IPV4);
+
 	/**
 	 * Create a UDP socket and bind it to a specific interface
 	 * and port address so that other UDP sockets on remote
@@ -717,8 +782,10 @@ public:
 	 * @param port number to deliver packets to.
 	 */
 	void setPeer(const IPV4Host &host, tpport_t port);
+	void connect(const IPV4Host &host, tpport_t port);
 #ifdef	CCXX_IPV6
 	void setPeer(const IPV6Host &host, tpport_t port);
+	void connect(const IPV6Host &host, tpport_t port);
 #endif
 
         /**
@@ -748,18 +815,17 @@ public:
 	 * @param len of packet buffer to send.
 	 * @return number of bytes sent.
 	 */
-	inline ssize_t send(const void *buf, size_t len)
-		{return IORET ::sendto(so, (const char*)buf, IOLEN len, 0, (struct sockaddr *)&peer, (socklen_t)sizeof(peer));};
+	ssize_t send(const void *buf, size_t len);
 
 	/**
 	 * Receive a message from any host.
 	 *
 	 * @param buf pointer to packet buffer to receive.
 	 * @param len of packet buffer to receive.
+	 * @param save sender address as peer to send reply if true.
 	 * @return number of bytes received.
 	 */
-	inline ssize_t receive(void *buf, size_t len)
-		{return IORET ::recv(so, (char *)buf, IOLEN len, 0);};
+	ssize_t receive(void *buf, size_t len, bool reply = false);
 
 	/**
 	 * Examine address of sender of next waiting packet.  This also
@@ -785,7 +851,13 @@ public:
 	 * @return number of bytes examined.
 	 */
 	inline ssize_t peek(void *buf, size_t len)
-		{return IORET ::recv(so, (char *)buf, IOLEN len, MSG_PEEK);};
+		{return _IORET64 ::recv(so, (char *)buf, _IOLEN64 len, MSG_PEEK);};
+
+	/**
+	 * Associate socket with a named connection
+	 */
+	void setPeer(const char *service);
+	void connect(const char *service);
 
 	/**
          * Disassociate this socket from any host connection.  No data
@@ -916,7 +988,7 @@ protected:
 	 * @param len of bytes to send.
 	 */
 	inline ssize_t send(const void *buf, size_t len)
-		{return IORET ::send(so, (const char *)buf, IOLEN len, 0);}
+		{return _IORET64 ::send(so, (const char *)buf, _IOLEN64 len, 0);}
 
 	/**
 	 * Stop transmitter.
@@ -949,7 +1021,7 @@ public:
 	 * @param len of bytes to send.
 	 */
 	inline ssize_t transmit(const char *buffer, size_t len)
-		{return IORET ::send(so, buffer, IOLEN len, MSG_DONTWAIT);}
+		{return _IORET64 ::send(so, buffer, _IOLEN64 len, MSG_DONTWAIT);}
 
 	/**
 	 * See if output queue is empty for sending more packets.
@@ -1060,7 +1132,7 @@ public:
 	 * @param len size of data receive buffer.
 	 */
 	inline ssize_t receive(void *buf, size_t len)
-		{return IORET ::recv(so, (char *)buf, IOLEN len, 0);};
+		{return _IORET64 ::recv(so, (char *)buf, _IOLEN64 len, 0);};
 
 	/**
 	 * See if input queue has data packets available.
@@ -1148,6 +1220,10 @@ public:
 class __EXPORT TCPSocket : protected Socket
 {
 protected:
+	int segsize;
+	void setSegmentSize(unsigned mss);
+
+public:
 	/**
 	 * A method to call in a derived TCPSocket class that is acting
 	 * as a server when a connection request is being accepted.  The
@@ -1162,12 +1238,18 @@ protected:
 	virtual bool onAccept(const IPV4Host &ia, tpport_t port)
 		{return true;}
 
-	friend class TCPStream;
-	friend class SocketPort;
-	friend class tcpstream;
-	friend class SimpleTCPStream; // Added by Mark S. Millard
+	/**
+	 * Fetch out the socket.
+	 */
+	inline SOCKET getSocket(void)
+		{return so;};
+
+	/**
+	 * Get the buffer size for servers.
+	 */
+	inline int getSegmentSize(void)
+		{return segsize;};
 
-public:
 	/**
 	 * A TCP "server" is created as a TCP socket that is bound
 	 * to a hardware address and port number on the local machine
@@ -1178,8 +1260,20 @@ public:
 	 * @param bind local ip address or interface to use.
 	 * @param port number to bind socket under.
 	 * @param backlog size of connection request queue.
+	 * @param buffer size for accepted streams.
 	 */
-	TCPSocket(const IPV4Address &bind, tpport_t port, int backlog = 5);
+	TCPSocket(const IPV4Address &bind, tpport_t port, unsigned backlog = 5, unsigned mss = 536);
+
+	/**
+	 * Create a named tcp socket by service and/or interface id.  
+	 * For IPV4 we use [host:]svc or [host/]svc for the string.
+	 * If we have getaddrinfo, we use that to obtain the addr to
+	 * bind for.
+	 *
+	 * @param name of host interface and service port to bind.
+	 * @param backlog size of connection request queue.
+	 */
+	TCPSocket(const char *service, unsigned backlog = 5, unsigned mss = 536);
 	
 	/**
 	 * Return address and port of next connection request.  This
@@ -1245,7 +1339,11 @@ public:
  */
 class __EXPORT TCPV6Socket : protected Socket
 {
-protected:
+private:
+	int segsize;
+	void setSegmentSize(unsigned mss);
+
+public:
 	/**
 	 * A method to call in a derived TCPSocket class that is acting
 	 * as a server when a connection request is being accepted.  The
@@ -1260,12 +1358,15 @@ protected:
 	virtual bool onAccept(const IPV6Host &ia, tpport_t port)
 		{return true;}
 
-	friend class TCPStream;
-	friend class SocketPort;
-	friend class tcpstream;
-	friend class SimpleTCPStream; // Added by Mark S. Millard
+	/**
+	 * Fetch out the socket.
+	 */
+	inline SOCKET getSocket(void)
+		{return so;};
+
+	inline int getSegmentSize(void)
+		{return segsize;};
 
-public:
 	/**
 	 * A TCP "server" is created as a TCP socket that is bound
 	 * to a hardware address and port number on the local machine
@@ -1277,7 +1378,18 @@ public:
 	 * @param port number to bind socket under.
 	 * @param backlog size of connection request queue.
 	 */
-	TCPV6Socket(const IPV6Address &bind, tpport_t port, int backlog = 5);
+	TCPV6Socket(const IPV6Address &bind, tpport_t port, unsigned backlog = 5, unsigned mss = 536);
+
+	/**
+	 * Create a TCP server for a named host interface and service
+	 * port.  We use [host/]port for specifying the optional host
+	 * name and service port since ':' is a valid char for ipv6
+	 * addresses.
+	 *
+	 * @param name of host interface and service to use.
+	 * @param backlog size of connection request queue.
+	 */
+	TCPV6Socket(const char *service, unsigned backlog = 5, unsigned mss = 536);
 	
 	/**
 	 * Return address and port of next connection request.  This
@@ -1345,44 +1457,44 @@ c:\projects\libraries\cplusplus\commonc++\win32\socket.h(358) : warning C4275: n
 class __EXPORT TCPStream : protected std::streambuf, public Socket, public std::iostream
 {
 private:
-	inline Error setBroadcast(bool enable)
-		{return Socket::setBroadcast(enable);};
-
-	inline IPV4Host getSender(tpport_t *port) const
-		{return IPV4Host();}
-
-	inline IPV4Host getIPV4Sender(tpport_t *port) const
-		{return IPV4Host();}
-
-#ifdef	CCXX_IPV6
-	inline IPV6Host getIPV6Sender(tpport_t *port) const
-		{return IPV6Host();}
-#endif
-
 	int doallocate();
 
+	void segmentBuffering(unsigned mss);
+
 	friend TCPStream& crlf(TCPStream&);
 	friend TCPStream& lfcr(TCPStream&);
 
 protected:
 	timeout_t timeout;
-	int bufsize;
+	size_t bufsize;
 	Family family;
 	char *gbuf, *pbuf;
 
+public:
 	/**
-	 * The constructor required for "tcpstream", a more C++ style
-	 * version of the TCPStream class.
+	 * The constructor required for building other classes or to
+	 * start an unconnected TCPStream for connect.
 	 */
-	TCPStream(Family family = IPV4, bool throwflag = true);
+	TCPStream(Family family = IPV4, bool throwflag = true, timeout_t to = 0);
 
+	/**
+	 * Disconnect the current session and prepare for a new one.
+	 */
+	void disconnect(void);
+
+	/**
+	 * Get protocol segment size.
+	 */
+	int getSegmentSize(void);
+
+protected:
 	/**
 	 * Used to allocate the buffer space needed for iostream
 	 * operations.  This function is called by the constructor.
 	 *
 	 * @param size of stream buffers from constructor.
 	 */
-	void allocate(int size);
+	void allocate(size_t size);
 
 	/**
 	 * Used to terminate the buffer space and cleanup the socket
@@ -1425,11 +1537,20 @@ protected:
 	 * @param port number to connect.
 	 * @param size of streaming input and output buffers.
 	 */
-	void connect(const IPV4Host &host, tpport_t port, int size);
+	void connect(const IPV4Host &host, tpport_t port, unsigned mss = 536);
 #ifdef	CCXX_IPV6
-	void connect(const IPV6Host &host, tpport_t port, int size);
+	void connect(const IPV6Host &host, tpport_t port, unsigned mss = 536);
 #endif
 
+	/**
+	 * Connect a TCP stream to a named destination host and port
+	 * number, using getaddrinfo interface if available.
+	 *
+	 * @param name of host and service to connect
+	 * @param size of stream buffer
+	 */
+	void connect(const char *service, unsigned mss = 536);
+
 	/**
 	 * Used in derived classes to refer to the current object via
 	 * it's iostream.  For example, to send a set of characters
@@ -1451,9 +1572,19 @@ public:
 	 * @param throwflag flag to throw errors.
 	 * @param timeout for all operations.
 	 */
-	TCPStream(TCPSocket &server, int size = 512, bool throwflag = true, timeout_t timeout = 0);
+	TCPStream(TCPSocket &server, bool throwflag = true, timeout_t timeout = 0);
 #ifdef	CCXX_IPV6
-	TCPStream(TCPV6Socket &server, int size = 512, bool throwflag = true, timeout_t timeout = 0);
+	TCPStream(TCPV6Socket &server, bool throwflag = true, timeout_t timeout = 0);
+#endif
+
+	/**
+	 * Accept a connection from a TCP Server.
+	 *
+	 * @param server socket listening
+	 */
+	void connect(TCPSocket &server);
+#ifdef	CCXX_IPV6
+	void connect(TCPV6Socket &server);
 #endif
 
 	/**
@@ -1466,11 +1597,22 @@ public:
 	 * @param throwflag flag to throw errors.
 	 * @param timeout for all operations.
 	 */
-	TCPStream(const IPV4Host &host, tpport_t port, int size = 512, bool throwflag = true, timeout_t timeout = 0);
+	TCPStream(const IPV4Host &host, tpport_t port, unsigned mss = 536, bool throwflag = true, timeout_t timeout = 0);
 #ifdef	CCXX_IPV6
-	TCPStream(const IPV6Host &host, tpport_t port, int size = 512, bool throwflag = true, timeout_t timeout = 0);
+	TCPStream(const IPV6Host &host, tpport_t port, unsigned mss = 536, bool throwflag = true, timeout_t timeout = 0);
 #endif
 
+	/**
+	 * Construct a named TCP Socket connected to a remote machine.
+	 *
+	 * @param name of remote service.
+	 * @param family of protocol.
+	 * @param size of streaming buffers.
+	 * @param throwflag flag to throw errors.
+	 * @param timeout for all operations.
+	 */
+	TCPStream(const char *service, Family fam = IPV4, unsigned mss = 536, bool throwflag = false, timeout_t to = 0);
+
 	/**
 	 * Set the I/O operation timeout for socket I/O operations.
 	 *
@@ -1516,7 +1658,7 @@ public:
 	 * @return count of bytes sent.
 	 * @param format string
 	 */
-	int printf(const char *format, ...);
+	size_t printf(const char *format, ...);
 #endif
 
 	/**
@@ -1536,82 +1678,17 @@ public:
           * @return number of bytes examined.
           */
          inline ssize_t peek(void *buf, size_t len)
-                 {return IORET ::recv(so, (char *)buf, IOLEN len, MSG_PEEK);};
+                 {return _IORET64 ::recv(so, (char *)buf, _IOLEN64 len, MSG_PEEK);};
 
 	/**
 	 * Return the size of the current stream buffering used.
 	 *
 	 * @return size of stream buffers.
 	 */
-	int getBufferSize(void) const
+	inline size_t getBufferSize(void) const
 		{return bufsize;};
 };
 
-/**
- * A more natural C++ "tcpstream" class for use by non-threaded
- * applications.  This class behaves a lot more like fstream and
- * similar classes.
- *
- * @author David Sugar <dyfet@ostel.com>
- * @short C++ "fstream" style tcpstream class.
- */
-class __EXPORT tcpstream : public TCPStream
-{
-public:
-	// copy constructor (fix a BUG in msvc7 compiler)
-	tcpstream(const tcpstream &rhs):TCPStream(rhs) {};
-
-	/**
-	 * Construct an unopened "tcpstream" object.
-	 */
-	tcpstream();
-
-	/**
-	 * Construct and "open" (connect) the tcp stream to a remote
-	 * socket.
-	 *
-	 * @param addr string address in form addr:port.
-	 * @param buffer size for streaming (optional).
-	 */
-	tcpstream(const char *addr, int buffer = 512);
-
-	/**
-	 * Construct and "accept" (connect) the tcp stream through
-	 * a server.
-	 *
-	 * @param tcp socket to accept from.
-	 * @param buffer size for streaming (optional).
-	 */
-	tcpstream(TCPSocket &tcp, int buffer = 512);
-
-	/**
-	 * Open a tcp stream connection.  This will close the currently
-	 * active connection first.
-	 *
-	 * @param addr string address in form addr:port.
-	 * @param buffer size for streaming (optional)
-	 */
-	void open(const char *addr, int buffer = 512);
-
-	/**
-	 * Open a tcp stream connection by accepting a tcp socket.
-	 *
-	 * @param tcp socket to accept from.
-	 * @param buffer size for streaming (optional)
-	 */
-	void open(TCPSocket &tcp, int buffer = 512);
-
-	/**
-	 * Close the active tcp stream connection.
-	 */
-	void close(void);
-
-	/**
-	 * Test to see if stream is open.
-	 */
-	bool operator!() const;
-};		
-
 /**
  * The TCP session is used to primarily to represent a client connection
  * that can be managed on a seperate thread.  The TCP session also supports
@@ -1673,10 +1750,10 @@ public:
 	 * @param stack allocation needed on some platforms.
 	 */
 	TCPSession(const IPV4Host &host, 
-		tpport_t port, int size = 512, int pri = 0, int stack = 0);
+		tpport_t port, size_t size = 536, int pri = 0, size_t stack = 0);
 #ifdef	CCXX_IPV6
 	TCPSession(const IPV6Host &host,
-		tpport_t port, int size = 512, int pri = 0, int stack = 0);
+		tpport_t port, size_t size = 536, int pri = 0, size_t stack = 0);
 #endif
 
 	/**
@@ -1689,11 +1766,9 @@ public:
 	 * @param pri execution priority relative to parent.
 	 * @param stack allocation needed on some platforms.
 	 */
-	TCPSession(TCPSocket &server, int size = 512, 
-		int pri = 0, int stack = 0);
+	TCPSession(TCPSocket &server, int pri = 0, size_t stack = 0);
 #ifdef	CCXX_IPV6
-	TCPSession(TCPV6Socket &server, int size = 512,
-		int pri = 0, int stack = 0);
+	TCPSession(TCPV6Socket &server, int pri = 0, size_t stack = 0);
 #endif
 };
 
@@ -1758,7 +1833,7 @@ protected:
 	 * @param port number to connect.
 	 * @param size of streaming input and output buffers.
 	 */
-	void Connect(const IPV4Host &host, tpport_t port, int size);
+	void Connect(const IPV4Host &host, tpport_t port, size_t size);
 
 
 public:
@@ -1770,7 +1845,7 @@ public:
 	 * @param server bound server tcp socket.
 	 * @param size of streaming input and output buffers.
 	 */
-	SimpleTCPStream(TCPSocket &server, int size = 512);
+	SimpleTCPStream(TCPSocket &server, size_t size = 512);
 
 	/**
 	 * Create a TCP stream by connecting to a TCP socket (on
@@ -1780,7 +1855,7 @@ public:
 	 * @param port number to connect.
 	 * @param size of streaming input and output buffers.
 	 */
-	SimpleTCPStream(const IPV4Host &host, tpport_t port, int size = 512);
+	SimpleTCPStream(const IPV4Host &host, tpport_t port, size_t size = 512);
 
 	/**
 	 * A copy constructor creates a new stream buffer.
@@ -1853,339 +1928,6 @@ public:
 
 };
 
-class __EXPORT SocketService;
-
-/**
- * The socket port is an internal class which is attached to and then
- * serviced by a specific SocketService "object".  Derived versions of
- * this class offer specific functionality for specific protocols.  Both
- * Common C++ supporting frameworks and application objects may be derived
- * from related protocol specific base classes.
- * 
- * A special set of classes, "SocketPort" and "SocketService", exist
- * for building realtime streaming media servers on top of UDP and TCP
- * protocols.  The "SocketPort" is used to hold a connected or associated TCP
- * or UDP socket which is being "streamed" and which offers callback methods
- * that are invoked from a "SocketService" thread.  SocketService's can be
- * pooled into logical thread pools that can service a group of SocketPorts.
- * A millisecond accurate "timer" is associated with each SocketPort and can
- * be used to time synchronize SocketPort I/O operations.
- *
- * @author David Sugar <dyfet@ostel.com>
- * @short base class for realtime and thread pool serviced protocols.
- */
-class __EXPORT SocketPort : public Socket, public TimerPort
-{
-private:
-	SocketPort *next, *prev;
-	SocketService *service;
-#ifndef WIN32
-	struct timeval porttimer;
-#ifdef USE_POLL
-	struct pollfd	* ufd;
-#endif
-#else
-	HANDLE event;
-#endif
-	bool detect_pending;
-	bool detect_output;
-	bool detect_disconnect;
-	
-	friend class SocketService;
-
-protected:
-	/** 
-	 * Construct an accepted TCP socket connection from a specific
-	 * bound TCP server.  This is meant to derive advanced application
-	 * specific TCP servers that can be thread pooled.
-	 *
-	 * @param svc pool thread object.
-	 * @param tcp socket object to accept.
-	 */
-	SocketPort(SocketService *svc, TCPSocket &tcp);
-#ifdef	CCXX_IPV6
-	SocketPort(SocketService *svc, TCPV6Socket &tcp);
-#endif
-
-	/**
-	 * Construct a bound UDP socket for use in deriving realtime
-	 * UDP streaming protocols handled by thread pool objects.
-	 *
-	 * @param svc pool thread object.
-	 * @param ia address of interface to bind.
-	 * @param port number to bind to.
-	 */
-	SocketPort(SocketService *svc, const IPV4Address &ia, tpport_t port);
-#ifdef	CCXX_IPV6
-	SocketPort(SocketService *svc, const IPV6Address &ia, tpport_t port);
-#endif
-	
-	/**
-	 * A non-blocking constructor for outbound tcp connections.
-	 * To detect when the connection is established, overload
-	 * SocketPort::output().  SocketPort::output() get's called by
-	 * the SocketService when the connection is ready,
-	 * SocketPort::disconnect() when the connect failed.  at the
-	 * moment you should set the socket state to "CONNECTED" when
-	 * SocketPort::output() get's called for the first time.
-	 *
-	 * @param svc pool thread object.
-	 * @param ih addess to connect to.
-	 * @param port number to connect to.
-	 **/
-	SocketPort(SocketService *svc, const IPV4Host &ih, tpport_t port);
-#ifdef	CCXX_IPV6
-	SocketPort(SocketService *svc, const IPV6Host &ih, tpport_t port);
-#endif
-
-	/** 
-	 * Attach yourself to the service pool thread object. The later version.
-	 *
-	 * @param svc pool thread object 
-	 */	 
-	 void attach( SocketService* svc );
-
-
-	/**
-	 * Disconnect the socket from the service thread pool and
-	 * the remote connection.
-	 */	
-	virtual ~SocketPort();
-
-	/**
-	 * Used to indicate if the service thread should monitor pending
-	 * data for us.
-	 */
-	void setDetectPending( bool );
-	
-	/**
-	 * Get the current state of the DetectPending flag.
-	 */
-	bool getDetectPending( void ) const
-		{ return detect_pending; }
-	
-	/**
-	 * Used to indicate if output ready monitoring should be performed
-	 * by the service thread.
-	 */
-	void setDetectOutput( bool );
-	
-	/**
-	 * Get the current state of the DetectOutput flag.
-	 */
-	bool getDetectOutput( void ) const
-		{ return detect_output; }
-
-	/**
-	 * Called by the service thread pool when the objects timer
-	 * has expired.  Used for timed events.
-	 */
-	virtual void expired(void)
-		{return;};
-
-	/**
-	 * Called by the service thread pool when input data is pending
-	 * for this socket.
-	 */
-	virtual void pending(void)
-		{return;};
-
-	/**
-	 * Called by the service thread pool when output data is pending
-	 * for this socket.
-	 */
-	virtual void output(void)
-		{return;};
-
-	/**
-	 * Called by the service thread pool when a disconnect has
-	 * occured.
-	 */
-	virtual void disconnect(void)
-		{return;};
-
-	/**
-	 * Connect a Socket Port to a known peer host.  This is normally
-	 * used with the UDP constructor.  This is also performed as a
-	 * non-blocking operation under Posix systems to prevent delays
-	 * in a callback handler.
-	 *
-	 * @return 0 if successful.
-	 * @param ia address of remote host or subnet.
-	 * @param port number of remote peer(s).
-	 */
-	Error connect(const IPV4Address &ia, tpport_t port);
-#ifdef	CCXX_IPV6
-	Error connect(const IPV6Address &ia, tpport_t port);
-#endif
-
-	/**
-	 * Transmit "send" data to a connected peer host.  This is not
-	 * public by default since an overriding protocol is likely to
-	 * be used in a derived class.
-	 *
-	 * @return number of bytes sent.
-	 * @param buf address of buffer to send.
-	 * @param len of bytes to send.
-	 */
-	inline ssize_t send(const void *buf, size_t len)
-		{return IORET ::send(so, (const char *)buf, IOLEN len, 0);};
-
-	/**
-	 * Receive a message from any host.  This is used in derived
-	 * classes to build protocols.
-	 *
-	 * @param buf pointer to packet buffer to receive.
-	 * @param len of packet buffer to receive.
-	 * @return number of bytes received.
-	 */
-	inline ssize_t receive(void *buf, size_t len)
-		{return IORET ::recv(so, (char *)buf, IOLEN len, 0);};
-
-	/**
-	 * Examine the content of the next packet.  This can be used
-	 * to build "smart" line buffering for derived TCP classes.
-	 *
-	 * @param buf pointer to packet buffer to examine.
-	 * @param len of packet buffer to examine.
-	 * @return number of bytes actually available.
-	 */
-	inline ssize_t peek(void *buf, size_t len)
-		{return IORET ::recv(so, (char *)buf, IOLEN len, MSG_PEEK);};
-
-public:
-	/**
-	 * Derived setTimer to notify the service thread pool of change
-	 * in expected timeout.  This allows SocketService to 
-	 * reschedule all timers.  Otherwise same as TimerPort.
-	 *
-	 * @param timeout in milliseconds.
-	 */
-	void setTimer(timeout_t timeout = 0);
-
-	/**
-	 * Derived incTimer to notify the service thread pool of a
-	 * change in expected timeout.  This allows SocketService to
-	 * reschedule all timers.  Otherwise same as TimerPort.
-	 *
-	 * @param timeout in milliseconds.
-	 */
-	void incTimer(timeout_t timeout);
-};
-
-/**
- * The SocketService is a thread pool object that is meant to service
- * attached socket ports.  Multiple pool objects may be created and
- * multiple socket ports may be attached to the same thread of execution.
- * This allows one to balance threads and sockets they service rather than
- * either using a single thread for all connections or a seperate thread
- * for each connection.  Features can be added through supported virtual
- * methods.
- *
- * @author David Sugar <dyfet@ostel.com>
- * @short Thread pool service object for socket ports.
- */
-class __EXPORT SocketService : public Thread, private Mutex
-{
-private:
-#ifndef WIN32
-	fd_set connect;
-	int iosync[2];
-	int hiwater;
-#else
-	// private syncronization class
-	class Sync;
-	Sync* sync;
-#endif
-	int volatile count;
-	SocketPort* volatile first, *last;
-
-	/**
-	 * Attach a new socket port to this service thread.
-	 *
-	 * @param port of SocketPort derived object to attach.
-	 */
-	void attach(SocketPort *port);
-	/**
-	 * Detach a socket port from this service thread.
-	 *
-	 * @param port of SocketPort derived object to remove.
-	 */
-	void detach(SocketPort *port);
-	
-	/**
-	 * The service thread itself.
-	 */
-	void run(void);
-
-	friend class SocketPort;
-
-protected:
-	/**
-	 * Handles all requests other than "termination".
-	 *
-	 * @param buf request id as posted from update().
-	 */
-	virtual void onUpdate(unsigned char buf)
-		{return;};
-
-	/**
-	 * Called once each time the service thread is rescheduled.
-	 * This is called after the mutex is locked and can be used to
-	 * slip in additional processing.
-	 */
-	virtual void onEvent(void)
-		{return;};
-
-	/**
-	 * Called for each port that is being processed in response to
-	 * an event.  This can be used to add additional notification
-	 * options during callback in combination with update().
-	 *
-	 * @param port SocketPort who's callback events are being evaluated.
-	 */
-	virtual void onCallback(SocketPort *port)
-		{return;};
-
-public:
-	/**
-	 * Notify service thread that a port has been added or
-	 * removed, or a timer changed, so that a new schedule
-	 * can be computed for expiring attached ports.  A "0"
-	 * is used to terminate the service thread, and additional
-	 * values can be specified which will be "caught" in the
-	 * onUpdate() handler.
-	 *
-	 * @param flag update flag value.
-	 */
-	void update(unsigned char flag = 0xff);
-
-	/**
-	 * Create a service thread for attaching socket ports.  The
-	 * thread begins execution with the first attached socket.
-	 *
-	 * @param pri of this thread to run under.
-	 * @param stack stack size.
-	 * @param id thread ID.
-	 */
-	SocketService(int pri = 0, size_t stack = 0, const char *id = NULL);
-
-	/**
-	 * Terminate the thread pool and eliminate any attached
-	 * socket ports.
-	 */
-	virtual ~SocketService();
-
-	/**
-	 * Get current reference count.  This can be used when selecting
-	 * the least used service handler from a pool.
-	 *
-	 * @return count of active ports.
-	 */
-	inline int getCount(void) const
-		{return count;};
-};
-
 #ifdef	COMMON_STD_EXCEPTION
 class __EXPORT SockException : public IOException
 {
