@@ -58,6 +58,8 @@
 namespace ost {
 #endif
 
+class MemPager;
+
 /**
  * This is a generic and portable string class.  It uses optimized
  * memory allocation strategies to efficiently handle smaller string
@@ -81,7 +83,9 @@ protected:
 	static const unsigned slotsize;
 	static const unsigned pagesize;
 	static const unsigned slotlimit;
-	static const unsigned slots;
+	static const unsigned slotcount;
+
+	friend class StringObject;
 
 private:
 	friend class MemPager;
@@ -165,7 +169,7 @@ protected:
 	 * @return string pointer to space.
 	 * @param size of space needed.
 	 */
-	char *getSpace(size_t size);
+	static char *getSpace(size_t size);
 
 	/**
 	 * Set the size of allocated space in the string variable
@@ -272,7 +276,7 @@ public:
 	 * @param count size of new string.
 	 * @param fill char to fill string with.
 	 */
-	String(size_t count, char fill = ' ');
+	String(size_t count, const char fill = ' ');
 
 	/**
 	 * Destroy the string...
@@ -864,6 +868,29 @@ public:
 	~SString();
 };
 
+/**
+ * The StringObject class is used to derive subclasses that use the
+ * String managed memory pool for all space allocations by overriding
+ * new and delete operators.  Due to size limits, StringObject should
+ * not hold very large objects.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short Objects managed in reusable String memory pools
+ */
+class __EXPORT StringObject
+{
+public:
+	/**
+	 * Create a new object in string managed space.
+	 */
+	void *operator new(size_t size) NEW_THROWS;
+
+	/**
+	 * Delete object from string managed space.
+	 */
+	void operator delete(void *obj);
+};
+
 #ifdef	CCXX_NAMESPACES
 }
 #endif
