@@ -80,6 +80,9 @@ namespace ost {
  	friend class SocketPort;
  	friend class unixstream;
 
+	void close(void);
+	char *path;
+
  public:
  	/**
  	 * A Unix domain "server" is created as a Unix domain socket that is bound
@@ -101,7 +104,7 @@ namespace ost {
  	 * Use base socket handler for ending this socket.
  	 */
  	virtual ~UnixSocket()
- 		{endSocket();};
+		{close();};
  };
 
  /**
