@@ -39,14 +39,9 @@
 // If you do not wish that, delete this exception notice.
 
 #include <cc++/config.h>
-#ifdef	CCXX_WITHOUT_EXTRAS
 #include <cc++/export.h>
-#endif
 #include <cc++/exception.h>
 #include <cc++/thread.h>
-#ifndef	CCXX_WITHOUT_EXTRAS
-#include <cc++/export.h>
-#endif
 #include <cc++/buffer.h>
 #include <cstdio>
 
@@ -54,6 +49,8 @@
 namespace ost {
 #endif
 
+const size_t Buffer::timeout = ((size_t)(-1l));
+
 #ifdef	WIN32
 Buffer::Buffer(size_t capacity) : Mutex()
 #else
@@ -78,14 +75,14 @@ Buffer::~Buffer()
 
 #ifdef	WIN32
 
-int Buffer::wait(void *buf, timeout_t timeout)
+size_t Buffer::wait(void *buf, timeout_t timeout)
 {
-	int	rc;
+	size_t	rc;
 
 	if(!timeout)
 			timeout = INFINITE;
 	if(Thread::waitThread(sem_head, timeout) != WAIT_OBJECT_0)
-		return -1;
+		return Buffer::timeout;
 	enterMutex();
 	rc = onWait(buf);
 	--_used;
@@ -94,15 +91,15 @@ int Buffer::wait(void *buf, timeout_t timeout)
 	return rc;
 }
 
-int Buffer::post(void *buf, timeout_t timeout)
+size_t Buffer::post(void *buf, timeout_t timeout)
 {
-	int	rc;
+	size_t	rc;
 
 	if(!timeout)
 			timeout = INFINITE;
 
 	if(Thread::waitThread(sem_tail, timeout) != WAIT_OBJECT_0)
-		return -1;
+		return Buffer::timeout;
 	enterMutex();
 	rc = onPost(buf);
 	++_used;
@@ -113,28 +110,28 @@ int Buffer::post(void *buf, timeout_t timeout)
 
 #else
 
-int Buffer::wait(void *buf, timeout_t timeout)
+size_t Buffer::wait(void *buf, timeout_t timeout)
 {
-	int rc = 0;
+	size_t rc = 0;
 	enterMutex();
 	while(!_used)
 	{
 		if(!Conditional::wait(timeout, true))
 		{
 			leaveMutex();
-			return -1;
+			return Buffer::timeout;
 		}
 	}
-	rc = onWait(buf);
+	rc = (ssize_t)onWait(buf);
 	--_used;
 	Conditional::signal(false);
 	leaveMutex();
 	return rc;
 }
 
-int Buffer::post(void *buf, timeout_t timeout)
+size_t Buffer::post(void *buf, timeout_t timeout)
 {
-	int rc = 0;
+	size_t rc = 0;
 
 	enterMutex();
 	while(_used == _size)
@@ -142,25 +139,25 @@ int Buffer::post(void *buf, timeout_t timeout)
 		if(!Conditional::wait(timeout, true))
 		{
 			leaveMutex();
-			return -1;
+			return Buffer::timeout;
 		}
 	}
-	rc = onPost(buf);
+	rc = (ssize_t)onPost(buf);
 	++_used;
 	Conditional::signal(false);
 	leaveMutex();
 	return rc;
 }
 
-int Buffer::peek(void *buf)
+size_t Buffer::peek(void *buf)
 {
-	int rc;
+	size_t rc;
 
 	enterMutex();
 	if(!_used)
 	{
 		leaveMutex();
-		return -1;
+		return 0;
 	}
 	rc = onPeek(buf);
 	leaveMutex();
@@ -203,26 +200,26 @@ bool FixedBuffer::isValid(void)
 
 #define	MAXBUF	(buf + (getSize() * objsize))
 
-int FixedBuffer::onWait(void *data)
+size_t FixedBuffer::onWait(void *data)
 {
 	memcpy(data, head, objsize);
 	if((head += objsize) >= MAXBUF)
 		head = buf;
-	return (int)objsize;
+	return objsize;
 }
 
-int FixedBuffer::onPost(void *data)
+size_t FixedBuffer::onPost(void *data)
 {
 	memcpy(tail, data, objsize);
 	if((tail += objsize) >= MAXBUF)
 		tail = buf;
-	return (int)objsize;
+	return objsize;
 }
 
-int FixedBuffer::onPeek(void *data)
+size_t FixedBuffer::onPeek(void *data)
 {
 	memcpy(data, head, objsize);
-	return (int)objsize;
+	return objsize;
 }
 
 ThreadQueue::ThreadQueue(const char *id, int pri, size_t stack) :
