@@ -39,10 +39,10 @@
 // If you do not wish that, delete this exception notice.
 
 #include <cc++/config.h>
+#include <cc++/exception.h>
+#include <cc++/thread.h>
 #include <cc++/export.h>
 #include <cc++/numbers.h>
-#include <cc++/thread.h>
-#include <cc++/exception.h>
 #include <cstdlib>
 #include <cstdio>
 #include <iostream>
@@ -55,7 +55,6 @@
 
 #ifdef	CCXX_NAMESPACES
 namespace ost {
-using std::string;
 using namespace std;
 #ifdef __BORLANDC__
 using std::time_t;
@@ -66,12 +65,11 @@ using std::localtime;
 
 Date::Date()
 {
-	time_t now;
-	struct tm *dt;
-	time(&now);
-	dt = localtime(&now);
+	time_t now = SysTime::getTime();
+	struct tm dt;
+       SysTime::getLocalTime(&now, &dt);
 
-	toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+	toJulian(dt.tm_year + 1900, dt.tm_mon + 1, dt.tm_mday);
 }
 
 Date::Date(struct tm *dt)
@@ -81,8 +79,9 @@ Date::Date(struct tm *dt)
 
 Date::Date(time_t tm)
 {
-	struct tm *dt = localtime(&tm);
-	toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+	struct tm dt;
+       SysTime::getLocalTime(&tm, &dt);
+	toJulian(dt.tm_year + 1900, dt.tm_mon + 1, dt.tm_mday);
 }
 
 Date::Date(char *str, size_t size)
@@ -92,8 +91,9 @@ Date::Date(char *str, size_t size)
 
 void Date::setDate(const char *str, size_t size)
 {
-	time_t now;
-	struct tm *dt;
+	time_t now = SysTime::getTime();
+	struct tm dt;
+       SysTime::getLocalTime(&now, &dt);
 	int year = 0;
 	const char *mstr = str;
 	const char *dstr = str;
@@ -106,9 +106,7 @@ void Date::setDate(const char *str, size_t size)
 #ifdef DEBUG
 		cout << "Date::SetDate(): 0000" << endl;
 #endif
-		time(&now);
-		dt = localtime(&now);
-		year = dt->tm_year + 1900;
+		year = dt.tm_year + 1900;
 		mstr = str;
 		dstr = str + 2;
 	}		
@@ -118,9 +116,7 @@ void Date::setDate(const char *str, size_t size)
 #ifdef DEBUG
 		cout << "Date::SetDate(): 00/00" << endl;
 #endif
-		time(&now);
-		dt = localtime(&now);
-		year = dt->tm_year + 1900;
+		year = dt.tm_year + 1900;
 		mstr = str;
 		dstr = str + 3;
 	}
@@ -130,10 +126,8 @@ void Date::setDate(const char *str, size_t size)
 #ifdef DEBUG
 		cout << "Date::SetDate(): 000000" << endl;
 #endif
-		time(&now);
-		dt = localtime(&now);
 		ZNumber nyear((char*)str, 2);
-		year = ((dt->tm_year + 1900) / 100) * 100 + nyear();
+		year = ((dt.tm_year + 1900) / 100) * 100 + nyear();
 		mstr = str + 2;
 		dstr = str + 4;
 	}
@@ -154,10 +148,8 @@ void Date::setDate(const char *str, size_t size)
 #ifdef DEBUG
 		cout << "Date::SetDate(): 00/00/00" << endl;
 #endif
-		time(&now);
-		dt = localtime(&now);
 		ZNumber nyear((char*)str, 2);
-		year = ((dt->tm_year + 1900) / 100) * 100 + nyear();
+		year = ((dt.tm_year + 1900) / 100) * 100 + nyear();
 		mstr = str + 3;
 		dstr = str + 6;
 	}
@@ -166,21 +158,25 @@ void Date::setDate(const char *str, size_t size)
 	else if(size == 10)
 	{
 #ifdef DEBUG
-		cout << "Date::SetDate(): 0000/00/00" << endl;
+		cout << "Date::SetDate(): 0000-00-00" << endl;
 #endif
 		ZNumber nyear((char*)str, 4);
 		year = nyear();
 		mstr = str + 5;
 		dstr = str + 8;
 	}
-	else if(getException() == Thread::throwObject)
+#ifdef	CCXX_EXCEPTIONS
+	else if(Thread::getException() == Thread::throwObject)
 	{
 		throw this;
 	}
-	else if(getException() == Thread::throwException)
+#ifdef	COMMON_STD_EXCEPTION
+	else if(Thread::getException() == Thread::throwException)
 	{
 		throw Exception("Date::setDate(): Invalid date.");
-	}	    	
+	}	    
+#endif
+#endif	
 	else {
 		julian = 0x7fffffffl;
 		return;
@@ -194,7 +190,7 @@ void Date::setDate(const char *str, size_t size)
 Date::Date(int year, unsigned month, unsigned day)
 {
 	toJulian(year, month, day);
-};
+}
 
 bool Date::isValid(void) const
 {
@@ -211,12 +207,12 @@ char *Date::getDate(char *buf) const
 
 time_t Date::getDate(tm* dt) const
 {
-	char buf[9];
+	char buf[11];
 	memset(dt, 0, sizeof(tm));
 	fromJulian(buf);
 	Number nyear(buf, 4);
-	Number nmonth(buf + 4, 2);
-	Number nday(buf + 6, 2);
+	Number nmonth(buf + 5, 2);
+	Number nday(buf + 8, 2);
 	
 	dt->tm_year = nyear() - 1900;
 	dt->tm_mon = nmonth() - 1;
@@ -233,7 +229,7 @@ time_t Date::getDate(void) const
 
 int Date::getYear(void) const
 {
-	char buf[9];
+	char buf[11];
 	fromJulian(buf);
 	Number num(buf, 4);
 	return num();
@@ -241,17 +237,17 @@ int Date::getYear(void) const
 
 unsigned Date::getMonth(void) const
 {
-	char buf[9];
+	char buf[11];
 	fromJulian(buf);
-	Number num(buf + 4, 2);
+	Number num(buf + 5, 2);
 	return num();
 }
 
 unsigned Date::getDay(void) const
 {
-	char buf[9];
+	char buf[11];
 	fromJulian(buf);
-	Number num(buf + 6, 2);
+	Number num(buf + 8, 2);
 	return num();
 }
 
@@ -260,21 +256,21 @@ unsigned Date::getDayOfWeek(void) const
 	return (unsigned)((julian + 1l) % 7l);
 }
 
-string Date::operator()() const
+String Date::operator()() const
 {
-	char buf[9];
+	char buf[11];
 
 	fromJulian(buf);
-	string date(buf);
+	String date(buf);
 
 	return date;
 }
 
 long Date::getValue(void) const
 {
-	char buf[9];
+	char buf[11];
 	fromJulian(buf);
-	return atol(buf);
+	return atol(buf) * 10000 + atol(buf + 5) * 100 + atol(buf + 8);
 }
 
 Date& Date::operator++()
@@ -291,7 +287,7 @@ Date& Date::operator--()
 	return *this;
 }
 
-Date& Date::operator+=(long val)
+Date& Date::operator+=(const long val)
 {
 	julian += val;
 	update();
@@ -305,32 +301,32 @@ Date& Date::operator-=(long val)
 	return *this;
 }
 
-int Date::operator==(Date &d)
+int Date::operator==(const Date &d)
 {
 	return julian == d.julian;
 }
 
-int Date::operator!=(Date &d)
+int Date::operator!=(const Date &d)
 {
 	return julian != d.julian;
 }
 
-int Date::operator<(Date &d)
+int Date::operator<(const Date &d)
 {
 	return julian < d.julian;
 }
 
-int Date::operator<=(Date &d)
+int Date::operator<=(const Date &d)
 {
 	return julian <= d.julian;
 }
 
-int Date::operator>(Date &d)
+int Date::operator>(const Date &d)
 {
 	return julian > d.julian;
 }
 
-int Date::operator>=(Date &d)
+int Date::operator>=(const Date &d)
 {
 	return julian >= d.julian;
 }
@@ -340,12 +336,16 @@ void Date::toJulian(long year, long month, long day)
 	julian = 0x7fffffffl;
 	
 	if(month < 1 || month > 12 || day < 1 || day > 31 || year == 0) {
-		if(getException() == Thread::throwObject) {
+#ifdef	CCXX_EXCEPTIONS
+		if(Thread::getException() == Thread::throwObject) {
 			throw this;
 		}
-		else if(getException() == Thread::throwException) {
+#ifdef	COMMON_STD_EXCEPTION
+		else if(Thread::getException() == Thread::throwException) {
 			throw Exception("Date::toJulian(): Invalid date.");
 		}
+#endif
+#endif
 		return;
 	}
 
@@ -364,8 +364,10 @@ void Date::fromJulian(char *buffer) const
 // Henry F. Fliegel and Thomas C. Van Flandern:
 
 	ZNumber nyear(buffer, 4);
-	ZNumber nmonth(buffer + 4, 2);
-	ZNumber nday(buffer + 6, 2);
+	buffer[4] = '-';
+	ZNumber nmonth(buffer + 5, 2);
+	buffer[7] = '-';
+	ZNumber nday(buffer + 8, 2);
 
 	double i, j, k, l, n;
 
@@ -383,10 +385,10 @@ void Date::fromJulian(char *buffer) const
 	nmonth = int(j);
 	nday = int(k);
 	
-	buffer[8] = '\0';
+	buffer[10] = '\0';
 }
 
-Date operator+(Date &date, long val)
+Date operator+(const Date &date, const long val)
 {
 	Date d = date;
 	d.julian += val;
@@ -394,7 +396,7 @@ Date operator+(Date &date, long val)
 	return d;
 }
 
-Date operator+(long val, Date &date)
+Date operator+(const long val, const Date &date)
 {
 	Date d = date;
 	d.julian += val;
@@ -402,7 +404,7 @@ Date operator+(long val, Date &date)
 	return d;
 }
 
-Date operator-(Date &date, long val)
+Date operator-(const Date &date, const long val)
 {
 	Date d = date;
 	d.julian -= val;
@@ -410,7 +412,7 @@ Date operator-(Date &date, long val)
 	return d;
 }
 
-Date operator-(long val, Date &date)
+Date operator-(const long val, const Date &date)
 {
 	Date d = date;
 	d.julian -= val;
@@ -420,12 +422,11 @@ Date operator-(long val, Date &date)
 
 Time::Time()
 {
-	time_t now;
-	struct tm *dt;
-	time(&now);
-	dt = localtime(&now);
+	time_t now = SysTime::getTime();
+	struct tm dt;
+       SysTime::getLocalTime(&now, &dt);
 
-	toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+	toSeconds(dt.tm_hour, dt.tm_min, dt.tm_sec);
 }
 
 Time::Time(struct tm *dt)
@@ -435,8 +436,9 @@ Time::Time(struct tm *dt)
 
 Time::Time(time_t tm)
 {
-	struct tm *dt = localtime(&tm);
-	toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+	struct tm dt;
+       SysTime::getLocalTime(&tm, &dt);
+	toSeconds(dt.tm_hour, dt.tm_min, dt.tm_sec);
 }
 
 Time::Time(char *str, size_t size)
@@ -464,18 +466,7 @@ char *Time::getTime(char *buf) const
 
 time_t Time::getTime(void) const
 {
-	char buf[7];
-	struct tm dt;
-	memset(&dt, 0, sizeof(dt));
-	fromSeconds(buf);
-	ZNumber nhour(buf, 2);
-	ZNumber nminute(buf + 2, 2);
-	ZNumber nsecond(buf + 4, 2);
-	
-	dt.tm_hour = nhour();
-	dt.tm_min = nminute();
-	dt.tm_sec = nsecond();
-	return mktime(&dt);
+    return static_cast<time_t>(seconds);
 }
 
 int Time::getHour(void) const
@@ -504,7 +495,7 @@ int Time::getSecond(void) const
 
 void Time::setTime(char *str, size_t size)
 {
-	int seconds = 00;
+	int sec = 00;
 
 	if(!size)
 		size = strlen(str);
@@ -512,39 +503,43 @@ void Time::setTime(char *str, size_t size)
 //00:00
 	if (size == 5)
 	{
-		seconds = 00;
+		sec = 00;
 	}
 //00:00:00	
 	else if (size == 8)
 	{
 		ZNumber nsecond(str + 6, 2);
-		seconds = nsecond();
+		sec = nsecond();
 	}
-	else if(getException() == Thread::throwObject)
+#ifdef	CCXX_EXCEPTIONS
+	else if(Thread::getException() == Thread::throwObject)
 	{
 		throw this;
 	}
-	else if(getException() == Thread::throwException)
+#ifdef	COMMON_STD_EXCEPTION
+	else if(Thread::getException() == Thread::throwException)
 	{
 		throw Exception("Time::setTime(): Invalid time.");
-	}	    	
+	}
+#endif
+#endif	    	
 	else {
 		return;
 	}
 
 	ZNumber nhour(str, 2);
 	ZNumber nminute(str+3, 2);
-	toSeconds(nhour(), nminute(), seconds);
+	toSeconds(nhour(), nminute(), sec);
 }
 
-string Time::operator()() const
+String Time::operator()() const
 {
 	char buf[7];
 
 	fromSeconds(buf);
-	string time(buf);
+	String strTime(buf);
 
-	return time;
+	return strTime;
 }
 
 long Time::getValue(void) const
@@ -566,46 +561,46 @@ Time& Time::operator--()
 	return *this;
 }
 
-Time& Time::operator+=(int val)
+Time& Time::operator+=(const int val)
 {
 	seconds += val;
 	Update();
 	return *this;
 }
 
-Time& Time::operator-=(int val)
+Time& Time::operator-=(const int val)
 {
 	seconds -= val;
 	Update();
 	return *this;
 }
 
-int Time::operator==(Time &t)
+int Time::operator==(const Time &t)
 {
 	return seconds == t.seconds;
 }
 
-int Time::operator!=(Time &t)
+int Time::operator!=(const Time &t)
 {
 	return seconds != t.seconds;
 }
 
-int Time::operator<(Time &t)
+int Time::operator<(const Time &t)
 {
 	return seconds < t.seconds;
 }
 
-int Time::operator<=(Time &t)
+int Time::operator<=(const Time &t)
 {
 	return seconds <= t.seconds;
 }
 
-int Time::operator>(Time &t)
+int Time::operator>(const Time &t)
 {
 	return seconds > t.seconds;
 }
 
-int Time::operator>=(Time &t)
+int Time::operator>=(const Time &t)
 {
 	return seconds >= t.seconds;
 }
@@ -615,12 +610,16 @@ void Time::toSeconds(int hour, int minute, int second)
 	seconds = -1;
 	
 	if (hour > 23 ||minute > 59 ||second > 59) {
-		if(getException() == Thread::throwObject) {
+#ifdef	CCXX_EXCEPTIONS
+		if(Thread::getException() == Thread::throwObject) {
 			throw this;
 		}
-		else if(getException() == Thread::throwException) {
+#ifdef	COMMON_STD_EXCEPTION
+		else if(Thread::getException() == Thread::throwException) {
 			throw Exception("Time::toSeconds(): Invalid time.");
 		}
+#endif
+#endif
 		return;
 	}
 
@@ -639,7 +638,7 @@ void Time::fromSeconds(char *buffer) const
 	buffer[6] = '\0';
 }
 
-Time operator+(Time &time1, Time &time2)
+Time operator+(const Time &time1, const Time &time2)
 {
 	Time t;
 	t.seconds = time1.seconds + time2.seconds;
@@ -647,7 +646,7 @@ Time operator+(Time &time1, Time &time2)
 	return t;
 }
 
-Time operator-(Time &time1, Time &time2)
+Time operator-(const Time &time1, const Time &time2)
 {
 	Time t;
 	t.seconds = time1.seconds - time2.seconds;
@@ -655,7 +654,7 @@ Time operator-(Time &time1, Time &time2)
 	return t;
 }
 
-Time operator+(Time &time, int val)
+Time operator+(const Time &time, const int val)
 {
 	Time t = time;
 	t.seconds += val;
@@ -663,7 +662,7 @@ Time operator+(Time &time, int val)
 	return t;
 }
 
-Time operator+(int val, Time &time)
+Time operator+(const int val, const Time &time)
 {
 	Time t = time;
 	t.seconds += val;
@@ -671,7 +670,7 @@ Time operator+(int val, Time &time)
 	return t;
 }
 
-Time operator-(Time &time, int val)
+Time operator-(const Time &time, const int val)
 {
 	Time t = time;
 	t.seconds -= val;
@@ -679,7 +678,7 @@ Time operator-(Time &time, int val)
 	return t;
 }
 
-Time operator-(int val, Time &time)
+Time operator-(const int val, const Time &time)
 {
 	Time t = time;
 	t.seconds -= val;
@@ -689,21 +688,26 @@ Time operator-(int val, Time &time)
 
 Datetime::Datetime(time_t tm)
 {
-	struct tm *dt = localtime(&tm);
-	toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
-	toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+	struct tm dt;
+       SysTime::getLocalTime(&tm, &dt);
+	toJulian(dt.tm_year + 1900, dt.tm_mon + 1, dt.tm_mday);
+	toSeconds(dt.tm_hour, dt.tm_min, dt.tm_sec);
 }
 
 Datetime::Datetime(tm *dt) :
 Date(dt), Time(dt)
 {}
 
-Datetime::Datetime(char *str, size_t size)
+Datetime::Datetime(const char *a_str, size_t size)
 {
 	char *timestr;
 
 	if (!size)
-		size = strlen(str);
+		size = strlen(a_str);
+
+	char *str = new char[size+1];
+	strncpy(str, a_str, size);
+	str[size]=0;
 
 // 00/00 00:00
 	if (size ==  11)
@@ -733,14 +737,21 @@ Datetime::Datetime(char *str, size_t size)
 		setDate(str, 10);
 		setTime(timestr,8);
 	}
-	else if(getException() == Thread::throwObject)
+#ifdef	CCXX_EXCEPTIONS
+	else if(Thread::getException() == Thread::throwObject)
 	{
+		delete str;
 		throw this;
 	}
-	else if(getException() == Thread::throwException)
+#ifdef	COMMON_STD_EXCEPTION
+	else if(Thread::getException() == Thread::throwException)
 	{
+		delete str;
 		throw Exception("Datetime::Datetime(): Invalid time.");
 	}	    	
+#endif
+#endif
+	delete str;
 }
 
 Datetime::Datetime(int year, unsigned month, unsigned day,
@@ -750,13 +761,12 @@ Datetime::Datetime(int year, unsigned month, unsigned day,
 
 Datetime::Datetime() : Date(), Time()
 {
-	time_t now;
-	struct tm *dt;
-	time(&now);
-	dt = localtime(&now);
+	time_t now = SysTime::getTime();
+	struct tm dt;
+       SysTime::getLocalTime(&now, &dt);
 
-	toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
-	toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+	toSeconds(dt.tm_hour, dt.tm_min, dt.tm_sec);
+	toJulian(dt.tm_year + 1900, dt.tm_mon + 1, dt.tm_mday);
 }
 
 bool Datetime::isValid(void) const
@@ -767,20 +777,21 @@ bool Datetime::isValid(void) const
 char *Datetime::getDatetime(char *buf) const
 {
 	fromJulian(buf);
-	fromSeconds(buf+8);
+	buf[10] = ' ';
+	fromSeconds(buf+11);
 	return buf;
 }
 
 time_t Datetime::getDatetime(void) const
 {
-	char buf[9];
+	char buf[11];
 	struct tm dt;
 	memset(&dt, 0, sizeof(dt));
 
 	fromJulian(buf);
 	ZNumber nyear(buf, 4);
-	ZNumber nmonth(buf + 4, 2);
-	ZNumber nday(buf + 6, 2);
+	ZNumber nmonth(buf + 5, 2);
+	ZNumber nday(buf + 8, 2);
 	dt.tm_year = nyear() - 1900;
 	dt.tm_mon = nmonth() - 1;
 	dt.tm_mday = nday();
@@ -797,7 +808,7 @@ time_t Datetime::getDatetime(void) const
 	return mktime(&dt);
 }
 
-Datetime& Datetime::operator=(Datetime datetime)
+Datetime& Datetime::operator=(const Datetime datetime)
 {
 	julian = datetime.julian;
 	seconds = datetime.seconds;
@@ -805,7 +816,7 @@ Datetime& Datetime::operator=(Datetime datetime)
 	return *this;	
 }
 
-Datetime& Datetime::operator+=(Datetime &datetime)
+Datetime& Datetime::operator+=(const Datetime &datetime)
 {
 	seconds += datetime.seconds;
 	julian += datetime.julian; 
@@ -813,7 +824,7 @@ Datetime& Datetime::operator+=(Datetime &datetime)
 	return *this;
 }
 
-Datetime& Datetime::operator-=(Datetime &datetime)
+Datetime& Datetime::operator-=(const Datetime &datetime)
 {
 	seconds -= datetime.seconds;
 	julian -= datetime.julian; 
@@ -821,14 +832,14 @@ Datetime& Datetime::operator-=(Datetime &datetime)
 	return *this;
 }
 
-Datetime& Datetime::operator+=(Time &time)
+Datetime& Datetime::operator+=(const Time &time)
 {
 	seconds += time.getValue();
 	Update();
 	return *this;
 }
 
-Datetime& Datetime::operator-=(Time &time)
+Datetime& Datetime::operator-=(const Time &time)
 {
 	seconds -= time.getValue();
 	Update();
@@ -836,17 +847,17 @@ Datetime& Datetime::operator-=(Time &time)
 }
 
 
-int Datetime::operator==(Datetime &d)
+int Datetime::operator==(const Datetime &d)
 {
 	return (julian == d.julian) && (seconds == d.seconds);
 }
 
-int Datetime::operator!=(Datetime &d)
+int Datetime::operator!=(const Datetime &d)
 {
 	return (julian != d.julian) || (seconds != d.seconds);
 }
 
-int Datetime::operator<(Datetime &d)
+int Datetime::operator<(const Datetime &d)
 {
 	if (julian != d.julian) {
 		return (julian < d.julian);
@@ -856,7 +867,7 @@ int Datetime::operator<(Datetime &d)
 	}
 }
 
-int Datetime::operator<=(Datetime &d)
+int Datetime::operator<=(const Datetime &d)
 {
 	if (julian != d.julian) {
 		return (julian < d.julian);
@@ -866,7 +877,7 @@ int Datetime::operator<=(Datetime &d)
 	}
 }
 
-int Datetime::operator>(Datetime &d)
+int Datetime::operator>(const Datetime &d)
 {
 	if (julian != d.julian) {
 		return (julian > d.julian);
@@ -876,7 +887,7 @@ int Datetime::operator>(Datetime &d)
 	}
 }
 
-int Datetime::operator>=(Datetime &d)
+int Datetime::operator>=(const Datetime &d)
 {
 	if (julian != d.julian) {
 		return (julian > d.julian);
@@ -886,38 +897,37 @@ int Datetime::operator>=(Datetime &d)
 	}
 }
 
-bool Datetime::operator!()
+bool Datetime::operator!() const
 {
 	return !(Date::isValid() && Time::isValid());
 }
 
 
-string Datetime::strftime(const char *format) const
+String Datetime::strftime(const char *format) const
 {
 	char buffer[64];
-	int last;
+	size_t last;
 	time_t t;
-	tm *tbp;
-	string retval;
+	tm tbp;
+	String retval;
 
 	t = getDatetime();
-        tbp = localtime(&t);
+       SysTime::getLocalTime(&t, &tbp);
 #ifdef	WIN32
-	last = ::strftime(buffer, 64, format, tbp);
+	last = ::strftime(buffer, 64, format, &tbp);
 #else
-	last = std::strftime(buffer, 64, format, tbp);
+	last = std::strftime(buffer, 64, format, &tbp);
 #endif
 
 	buffer[last] = '\0';
 	retval = buffer;
 	return retval;
-
 }
 
 DateNumber::DateNumber(char *str) :
 Number(str, 10), Date(str, 10)
-{};
+{}
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
