@@ -39,6 +39,9 @@
 // If you do not wish that, delete this exception notice.
 
 #include <cc++/config.h>
+#include <cc++/string.h>
+#include <cc++/exception.h>
+#include <cc++/thread.h>
 #include <cc++/export.h>
 #include <cc++/digest.h>
 
@@ -203,6 +206,6 @@ ostream& CRC32Digest::strDigest(ostream &os)
 
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif	
 	
