@@ -45,6 +45,7 @@
 #include "private.h"
 
 #ifdef WIN32
+#include <sys/stat.h>
 #include <malloc.h>
 #endif
 
@@ -283,6 +284,142 @@ const char *Dir::getName(void)
 #endif // WIN32
 }
 
+DirTree::DirTree(const char *prefix, unsigned depth)
+{
+	max = ++depth;
+	dir = new Dir[depth];
+	current = 0;
+	
+	open(prefix);
+}
+
+DirTree::DirTree(unsigned depth)
+{
+	max = ++depth;
+	dir = new Dir[depth];
+	current = 0;
+}
+
+void DirTree::open(const char *prefix)
+{
+	char *cp;
+
+	close();
+
+	if(!isDir(prefix))
+		return;
+
+	snprintf(path, sizeof(path), "%s/", prefix);
+	prefixpos = (unsigned)strlen(path) - 1;
+
+	while(NULL != (cp = strchr(path, '\\')))
+		*cp = '/';
+
+	while(prefixpos && path[prefixpos - 1] == '/')
+		path[prefixpos--] = 0;
+
+	dir[current++].open(prefix);	
+}
+
+DirTree::~DirTree()
+{
+	close();
+
+	if(dir)
+		delete[] dir;
+
+	dir = NULL;
+}
+
+unsigned DirTree::perform(const char *prefix)
+{
+	unsigned count = 0;
+
+	open(prefix);
+	while(NULL != (getPath()))
+		++count;
+	close();
+	return count;
+}
+
+void DirTree::close(void)
+{
+	while(current--)
+		dir[current].close();
+
+	current = 0;
+}
+
+
+bool DirTree::filter(const char *path, struct stat *ino)
+{
+	path = strrchr(path, '/');
+
+	if(path)
+		++path;
+	else
+		return false;
+
+	if(!strcmp(path, "."))
+		return false;
+
+	if(!strcmp(path, ".."))
+		return false;
+
+	if(!ino)
+		return false;
+
+	return true;
+}
+
+char *DirTree::getPath(void)
+{
+	char *cp;
+	const char *name;
+	struct stat ino;
+	bool flag;
+
+	while(current)
+	{
+		cp = strrchr(path, '/');
+		name = dir[current - 1].getName();
+		if(!name)
+		{
+			*cp = 0;
+			dir[--current].close();
+			continue;
+		}
+		snprintf(cp + 1, sizeof(path) - strlen(path) - 2, "%s", name);
+	
+		if(::stat(path, &ino))
+		{
+			ino.st_mode = 0;
+			flag = filter(path, NULL);
+		}
+		else
+			flag = filter(path, &ino);
+
+		if(!flag)
+			continue;
+
+		if((ino.st_mode & S_IFMT) == S_IFDIR)
+		{
+			if(!canAccess(path))
+				break;
+
+			if(current < max)
+				dir[current++].open(path);
+
+			snprintf(path + strlen(path), sizeof(path) - strlen(path), "/");
+		}
+		break;
+	}
+	if(!current)
+		return NULL;
+
+	return path;	
+}
+
 #ifdef	CCXX_NAMESPACES
 }
 #endif
