@@ -58,6 +58,7 @@
 #include <cc++/exception.h>
 #include <cc++/thread.h>
 #include <cc++/file.h>
+#include <cc++/process.h>
 #include "private.h"
 
 #ifdef	__BORLANDC__
@@ -106,6 +107,10 @@ enum
 
 #endif // ndef WIN32
 		
+#if defined(_OSF_SOURCE) && defined(_POSIX_C_SOURCE) && _POSIX_C_SOURCE > 1
+#undef	LOCK_EX
+#undef	LOCK_SH
+#endif
 
 #if 0
 /*
@@ -1003,6 +1008,16 @@ bool SharedFile::operator--(void)
 	return false;
 }
 
+size_t MappedFile::pageAligned(size_t size)
+{
+	size_t pages = size / Process::getPageSize();
+
+	if(size % Process::getPageSize())
+		++pages;
+
+	return pages * Process::getPageSize();
+}
+
 #ifdef WIN32
 
 static void makemapname(const char *source, char *target)
@@ -1019,6 +1034,49 @@ static void makemapname(const char *source, char *target)
 	*target = 0;
 }
 
+MappedFile::MappedFile(const char *fname, Access mode, size_t size) :
+RandomFile(fname)
+{
+	DWORD share, page;
+	map = INVALID_HANDLE_VALUE;
+	fcb.address = NULL;
+
+        switch(mode)
+        {
+        case accessReadOnly:
+                share = FILE_SHARE_READ;  
+                page = PAGE_READONLY;
+                prot = FILE_MAP_READ;
+                break;   
+        case accessWriteOnly:
+                share = FILE_SHARE_WRITE;
+                page = PAGE_WRITECOPY;
+                prot = FILE_MAP_COPY;
+                break;
+        case accessReadWrite:
+                share = FILE_SHARE_READ|FILE_SHARE_WRITE;
+                page = PAGE_READWRITE;
+                prot = FILE_MAP_WRITE;
+        }
+	fd = CreateFile(pathname, mode, share, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL); 
+	if(fd == INVALID_HANDLE_VALUE)
+	{
+		error(errOpenFailed);
+		return;
+	}
+	SetFilePointer(fd, (LONG)size, 0, FILE_BEGIN);
+	SetEndOfFile(fd);
+	makemapname(fname, mapname);
+	map = CreateFileMapping(fd, NULL, page, 0, 0, mapname);
+	if(!map)
+		error(errMapFailed);
+	fcb.address = MapViewOfFile(map, prot, 0, 0, size);
+	fcb.len = (ccxx_size_t)size;
+	fcb.pos = 0;
+	if(!fcb.address)
+		error(errMapFailed);
+}
+
 MappedFile::MappedFile(const char *fname, Access mode) :
 RandomFile(fname)
 {
@@ -1102,7 +1160,10 @@ RandomFile(fname)
 MappedFile::~MappedFile()
 {
 	if(fcb.address)
+	{
+		unlock();
 		UnmapViewOfFile(fcb.address);
+	}
 
 	if(map != INVALID_HANDLE_VALUE)
 		CloseHandle(map);
@@ -1121,14 +1182,20 @@ void MappedFile::sync(caddr_t address, size_t len)
 void MappedFile::release(caddr_t address, size_t len)
 {
 	if(fcb.address)
+	{
+		unlock();
 		UnmapViewOfFile(fcb.address);
+	}
 	fcb.address = NULL;
 }
 
 caddr_t MappedFile::fetch(off_t pos, size_t len)
 {
 	if(fcb.address)
+	{
+		unlock();
 		UnmapViewOfFile(fcb.address);
+	}
 
 	fcb.address = MapViewOfFile(map, prot, 0, pos, len);
 	fcb.len = (ccxx_size_t)len;
@@ -1138,6 +1205,7 @@ caddr_t MappedFile::fetch(off_t pos, size_t len)
 
 	return fcb.address;
 }
+
 void MappedFile::update(size_t offset, size_t len)
 {
 }
@@ -1146,6 +1214,26 @@ void MappedFile::update(caddr_t address, size_t len)
 {
 }
 
+bool MappedFile::lock(void)
+{
+	unlock();
+	if(VirtualLock(fcb.address, fcb.len))
+		fcb.locked = true;
+	return fcb.locked;
+}
+
+void MappedFile::unlock(void)
+{
+	if(!fcb.address)
+		fcb.locked = false;
+
+	if(!fcb.locked)
+		return;
+
+	VirtualUnlock(fcb.address, fcb.len);
+	fcb.locked = false;
+}
+
 #else
 MappedFile::MappedFile(const char *fname, Access mode) :
 RandomFile(fname)
@@ -1174,6 +1262,43 @@ RandomFile(fname)
 	}
 }
 
+MappedFile::MappedFile(const char *fname, Access mode, size_t size) :
+RandomFile(fname)
+{
+	fd = open(fname, (int)mode | O_CREAT, 0660);
+
+	if(fd < 0)
+	{
+		error(errOpenFailed);
+		return;
+	}
+
+	switch(mode)
+	{
+	case O_RDONLY:
+		prot = PROT_READ;
+		break;
+	case O_WRONLY:
+		prot = PROT_WRITE;
+		break;
+	default:
+		prot = PROT_READ | PROT_WRITE;
+	}
+
+	enterMutex();
+	lseek(fd, size, SEEK_SET);
+	fcb.address = (caddr_t)mmap(NULL, size, prot, MAP_SHARED, fd, 0);
+	fcb.len = size;
+	fcb.pos = 0;
+	leaveMutex();
+	if((caddr_t)(fcb.address) == (caddr_t)(MAP_FAILED))
+	{
+		close(fd);
+		fd = -1;
+		error(errMapFailed);
+	}
+}
+
 MappedFile::MappedFile(const char *fname, pos_t pos, size_t len, Access mode) :
 RandomFile(fname)
 {
@@ -1212,6 +1337,7 @@ RandomFile(fname)
 
 MappedFile::~MappedFile()
 {
+	unlock();
 	final();
 }
 
@@ -1229,6 +1355,9 @@ void MappedFile::release(caddr_t address, size_t len)
 	if(len)
 		fcb.len = len;
 
+	if(fcb.locked)
+		unlock();
+
 	munmap(fcb.address, fcb.len);
 	leaveMutex();
 }
@@ -1236,6 +1365,7 @@ void MappedFile::release(caddr_t address, size_t len)
 caddr_t MappedFile::fetch(off_t pos, size_t len)
 {
 	enterMutex();
+	unlock();
 	fcb.len = len;
 	fcb.pos = pos;
 	lseek(fd, fcb.pos + len, SEEK_SET);
@@ -1244,6 +1374,26 @@ caddr_t MappedFile::fetch(off_t pos, size_t len)
 	return fcb.address;
 }
 
+bool MappedFile::lock(void)
+{
+	unlock();
+	if(!mlock(fcb.address, fcb.len))
+		fcb.locked = true;
+	return fcb.locked;
+}
+
+void MappedFile::unlock(void)
+{
+	if(!fcb.address)
+		fcb.locked = false;
+
+	if(!fcb.locked)
+		return;
+
+	munlock(fcb.address, fcb.len);
+	fcb.locked = false;
+}
+
 void MappedFile::update(size_t offset, size_t len)
 {
 	int mode = MS_ASYNC;
@@ -1271,6 +1421,58 @@ void MappedFile::update(caddr_t address, size_t len)
 }
 #endif // ndef WIN32
 
+time_t lastAccessed(const char *path)
+{
+#ifdef	WIN32
+	__int64 ts;
+	WIN32_FILE_ATTRIBUTE_DATA ino;
+
+	if(!GetFileAttributesEx(path, GetFileExInfoStandard, &ino))
+		return 0;
+
+	ts = ((__int64)ino.ftLastAccessTime.dwHighDateTime << 32) +
+		ino.ftLastAccessTime.dwLowDateTime;
+
+	ts -= (SECS_BETWEEN_EPOCHS * SECS_TO_100NS);
+        ts /= SECS_TO_100NS;
+
+        return(time_t)ts;
+#else
+	struct stat ino;
+
+	if(stat(path, &ino))
+		return 0;
+
+	return ino.st_atime;
+#endif
+}
+
+time_t lastModified(const char *path)
+{
+#ifdef	WIN32
+	__int64 ts;
+	WIN32_FILE_ATTRIBUTE_DATA ino;
+
+	if(!GetFileAttributesEx(path, GetFileExInfoStandard, &ino))
+		return 0;
+
+	ts = ((__int64)ino.ftLastWriteTime.dwHighDateTime << 32) +
+		ino.ftLastWriteTime.dwLowDateTime;
+
+	ts -= (SECS_BETWEEN_EPOCHS * SECS_TO_100NS);
+	ts /= SECS_TO_100NS;
+
+	return(time_t)ts;
+#else
+	struct stat ino;
+	
+	if(stat(path, &ino))
+		return 0;
+
+	return ino.st_mtime;
+#endif
+}
+
 bool isDir(const char *path)
 {
 #ifdef WIN32
