@@ -40,7 +40,7 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
-#include <cc++/socket.h>
+#include <cc++/address.h>
 #include "private.h"
 #include <cstdlib>
 
@@ -49,28 +49,29 @@ namespace ost {
 #endif
 
 #ifndef WIN32
-Mutex InetAddress::mutex;
+Mutex IPV4Address::mutex;
 #endif
 
-const InetMcastAddrValidator InetMcastAddress::validator;
+const IPV4MulticastValidator IPV4Multicast::validator;
 
-void
-InetMcastAddrValidator::operator()(const in_addr address) const
+void IPV4MulticastValidator::operator()(const in_addr address) const
 {
+#ifdef	CCXX_EXCEPTIONS
 	// "0.0.0.0" is always accepted, as it is an "empty" address.
 	if ( (address.s_addr != INADDR_ANY) &&
 	     (address.s_addr & MCAST_VALID_MASK) != MCAST_VALID_VALUE ) {
 		throw "Multicast address not in the valid range: from 224.0.0.1 through 239.255.255.255";
 	}
+#endif
 }
 
-InetAddress::InetAddress(const InetAddrValidator *_validator) 
+IPV4Address::IPV4Address(const IPV4Validator *_validator) 
 	: validator(_validator), ipaddr(NULL), addr_count(0), hostname(NULL)
 {
 	*this = (long unsigned int)INADDR_ANY;
 }
 
-InetAddress::InetAddress(const char *address, const InetAddrValidator *_validator) : 
+IPV4Address::IPV4Address(const char *address, const IPV4Validator *_validator) : 
 	validator(_validator), ipaddr(NULL), addr_count(0), hostname(NULL)
 {
 	if ( this->validator )
@@ -81,7 +82,7 @@ InetAddress::InetAddress(const char *address, const InetAddrValidator *_validato
 		setAddress(address);
 }
 
-InetAddress::InetAddress(struct in_addr addr, const InetAddrValidator *_validator) : 
+IPV4Address::IPV4Address(struct in_addr addr, const IPV4Validator *_validator) : 
 	validator(_validator), ipaddr(NULL), hostname(NULL)
 {
 	if ( this->validator ){
@@ -93,33 +94,38 @@ InetAddress::InetAddress(struct in_addr addr, const InetAddrValidator *_validato
 	ipaddr[0] = addr;
 }
 
-InetAddress::InetAddress(const InetAddress &rhs) : validator(rhs.validator),
-						   addr_count(rhs.addr_count),
-						   hostname(NULL)
+IPV4Address::IPV4Address(const IPV4Address &rhs) : 
+validator(rhs.validator), addr_count(rhs.addr_count), hostname(NULL)
 {
 	ipaddr = new struct in_addr[addr_count];
 	memcpy(ipaddr, rhs.ipaddr, sizeof(struct in_addr) * addr_count);
 }
 
-InetAddress::~InetAddress()
+IPV4Address::~IPV4Address()
 {
 	if(ipaddr)
+	{
 		delete[] ipaddr;
+		ipaddr = NULL;
+	}
 	if(hostname)
-		delete[] hostname;
+	{
+		delString(hostname);
+		hostname = NULL;
+	}
 }
 
-struct in_addr InetAddress::getAddress(void) const
+struct in_addr IPV4Address::getAddress(void) const
 {
 	return ipaddr[0];
 }
 
-struct in_addr InetAddress::getAddress(size_t i) const
+struct in_addr IPV4Address::getAddress(size_t i) const
 {
 	return (i < addr_count ? ipaddr[i] : ipaddr[0]);
 }
 
-bool InetAddress::isInetAddress(void) const
+bool IPV4Address::isInetAddress(void) const
 {
 	struct in_addr addr;
 	memset(&addr, 0, sizeof(addr));
@@ -128,7 +134,7 @@ bool InetAddress::isInetAddress(void) const
 	return false;
 }
 
-InetAddress &InetAddress::operator=(const char *str)
+IPV4Address &IPV4Address::operator=(const char *str)
 {
 	if(str == 0 || !strcmp(str, "*"))
 		str = "0.0.0.0";
@@ -138,7 +144,7 @@ InetAddress &InetAddress::operator=(const char *str)
 	return *this;
 }
 
-InetAddress &InetAddress::operator=(struct in_addr addr)
+IPV4Address &IPV4Address::operator=(struct in_addr addr)
 {
 	if(ipaddr)
 		delete[] ipaddr;
@@ -148,12 +154,12 @@ InetAddress &InetAddress::operator=(struct in_addr addr)
 	ipaddr = new struct in_addr[1];
 	ipaddr[0] = addr;
 	if(hostname)
-		delete[] hostname;
+		delString(hostname);
 	hostname = NULL;
 	return *this;
 }
 
-InetAddress &InetAddress::operator=(unsigned long addr)
+IPV4Address &IPV4Address::operator=(unsigned long addr)
 {
 	if ( validator )
 		(*validator)(*reinterpret_cast<in_addr*>(&addr));
@@ -163,12 +169,12 @@ InetAddress &InetAddress::operator=(unsigned long addr)
 	ipaddr = new struct in_addr[1];
         memcpy(ipaddr, &addr, sizeof(struct in_addr));
 	if(hostname)
-		delete[] hostname;
+		delString(hostname);
 	hostname = NULL;
 	return *this;        
 }
 
-InetAddress &InetAddress::operator=(const InetAddress &rhs)
+IPV4Address &IPV4Address::operator=(const IPV4Address &rhs)
 {
 	if(this == &rhs) return *this;
 
@@ -179,15 +185,15 @@ InetAddress &InetAddress::operator=(const InetAddress &rhs)
 	memcpy(ipaddr, rhs.ipaddr, sizeof(struct in_addr) * addr_count);
 	validator = rhs.validator;
 	if(hostname)
-		delete[] hostname;
+		delString(hostname);
 	hostname = NULL;
 
 	return *this;
 }
 
-bool InetAddress::operator==(const InetAddress &a) const
+bool IPV4Address::operator==(const IPV4Address &a) const
 {
-	const InetAddress *smaller, *larger;
+	const IPV4Address *smaller, *larger;
 	size_t s, l;
 	
 	if(addr_count > a.addr_count)
@@ -213,13 +219,13 @@ bool InetAddress::operator==(const InetAddress &a) const
 	return true;
 }
 
-bool InetAddress::operator!=(const InetAddress &a) const
+bool IPV4Address::operator!=(const IPV4Address &a) const
 {
 	// Impliment in terms of operator==
 	return (*this == a ? false : true);
 }
 
-InetHostAddress &InetHostAddress::operator&=(const InetMaskAddress &ma)
+IPV4Host &IPV4Host::operator&=(const IPV4Mask &ma)
 {
 	for(size_t i = 0; i < addr_count; i++)
 	{		
@@ -231,17 +237,17 @@ InetHostAddress &InetHostAddress::operator&=(const InetMaskAddress &ma)
 			*(a++) &= *(m++);
 	}
 	if(hostname)
-		delete[] hostname;
+		delString(hostname);
 	hostname = NULL;
 
 	return *this;
 }
 	
-InetHostAddress::InetHostAddress(struct in_addr addr) :
-InetAddress(addr) {}
+IPV4Host::IPV4Host(struct in_addr addr) :
+IPV4Address(addr) {}
 
-InetHostAddress::InetHostAddress(const char *host) : 
-InetAddress(host)
+IPV4Host::IPV4Host(const char *host) : 
+IPV4Address(host)
 {
 	char namebuf[256];
 
@@ -252,14 +258,13 @@ InetAddress(host)
 	}
 }
 
-bool InetAddress::setIPAddress(const char *host)
+bool IPV4Address::setIPAddress(const char *host)
 {
 	if(!host)
 		return false;
 
 #if defined(WIN32)
-	struct sockaddr_in addr;
-	
+	struct sockaddr_in addr;	
 	addr.sin_addr.s_addr = inet_addr(host);
 	if ( validator )
 		(*validator)(addr.sin_addr);
@@ -279,10 +284,10 @@ bool InetAddress::setIPAddress(const char *host)
 	return true;
 }
 
-void InetAddress::setAddress(const char *host)
+void IPV4Address::setAddress(const char *host)
 {	
 	if(hostname)
-		delete[] hostname;
+		delString(hostname);
 	hostname = NULL;
 
 	if(!host)  // The way this is currently used, this can never happen
@@ -291,6 +296,14 @@ void InetAddress::setAddress(const char *host)
 		return;
 	}
 
+#ifdef	WIN32
+	if(!stricmp(host, "localhost"))
+	{
+		*this = (long unsigned int)inet_addr("127.0.0.1");
+		return;
+	}
+#endif
+
 	if(!setIPAddress(host))
 	{
 		struct hostent *hp;
@@ -348,12 +361,12 @@ void InetAddress::setAddress(const char *host)
 	}
 }
 
-BroadcastAddress::BroadcastAddress(const char *net) : 
-InetAddress(net)
+IPV4Broadcast::IPV4Broadcast(const char *net) : 
+IPV4Address(net)
 {
 }
 
-InetMaskAddress::InetMaskAddress(const char *mask)
+IPV4Mask::IPV4Mask(const char *mask)
 {
 	unsigned long x = 0xffffffff;
 	int l = 32 - atoi(mask);
@@ -363,15 +376,17 @@ InetMaskAddress::InetMaskAddress(const char *mask)
 
 	if(l < 1 || l > 32)
 	{
-		if(getException() == Thread::throwObject)
-			throw((InetAddress *)this);
+#ifdef	CCXX_EXCEPTIONS
+		if(Thread::getException() == Thread::throwObject)
+			throw((IPV4Address *)this);
+#endif
 		return;
 	}
 
 	*this = htonl(x << l);
 }
 
-const char *InetAddress::getHostname(void) const
+const char *IPV4Address::getHostname(void) const
 {
 	struct hostent *hp = NULL;
 	struct in_addr addr0;
@@ -380,6 +395,15 @@ const char *InetAddress::getHostname(void) const
 	if(!memcmp(&addr0, &ipaddr[0], sizeof(addr0)))
 		return NULL;
 
+#ifdef	WIN32
+	memset(&addr0, 0xff, sizeof(addr0));
+	if(!memcmp(&addr0, &ipaddr[0], sizeof(addr0)))
+		return "255.255.255.255";
+	long a = inet_addr("127.0.0.1");
+	if(!memcmp(&a, &ipaddr[0], sizeof(a)))
+		return "localhost";
+#endif
+
 #if defined(__GLIBC__)
 	char   hbuf[8192];
 	struct hostent hb;
@@ -400,37 +424,35 @@ const char *InetAddress::getHostname(void) const
 #endif
 	if(hp) {
 		if(hostname)
-			delete[] hostname;
-		hostname = new char[strlen(hp->h_name) + 1];
-		strcpy(hostname,hp->h_name);
+			delString(hostname);
+		hostname = newString(hp->h_name);
 		return hostname;
 	} else {
 		return inet_ntoa(ipaddr[0]);
 	}
 }
 	
-InetHostAddress operator&(const InetHostAddress &addr, 
-			  const InetMaskAddress &mask)
+IPV4Host operator&(const IPV4Host &addr, const IPV4Mask &mask)
 {
-	InetHostAddress temp = addr;
+	IPV4Host temp = addr;
 	temp &= mask;
 	return temp;
 }
 
-InetMcastAddress::InetMcastAddress() : 
-	InetAddress(&validator)
+IPV4Multicast::IPV4Multicast() : 
+	IPV4Address(&validator)
 { }
 
-InetMcastAddress::InetMcastAddress(const struct in_addr address) : 
-	InetAddress(address,&validator)
+IPV4Multicast::IPV4Multicast(const struct in_addr address) : 
+	IPV4Address(address,&validator)
 { }
 
-InetMcastAddress::InetMcastAddress(const char *address) :
-	InetAddress(address,&validator)
+IPV4Multicast::IPV4Multicast(const char *address) :
+	IPV4Address(address,&validator)
 { }
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
