@@ -46,6 +46,31 @@
 #include <cstdlib>
 #include <cstdio>
 
+#ifndef	CAPE_REGISTRY_APPLICATIONS
+#define	CAPE_REGISTRY_APPLICATIONS	"SOFTWARE\\CAPE Applications"
+#define	CAPE_REGISTRY_USERSETTINGS	"Software\\CAPE Applications"
+#endif
+
+#ifdef	WIN32
+class Registry
+{
+public:
+	const char *configdir;
+	Registry();
+};
+
+Registry::Registry()
+{
+	configdir = getenv("SystemRoot");
+	if(!configdir || !*configdir)
+		configdir = getenv("windir");
+	if(!configdir || !*configdir)
+		configdir = "C:\\WINDOWS";
+}
+
+static Registry registry;
+#endif
+
 #ifdef	CCXX_NAMESPACES
 namespace ost {
 using namespace std;
@@ -277,15 +302,89 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 {
 	// FIXME: use string of dinamic allocation
 	char path[KEYDATA_PATH_SIZE];
-	char buffer[256];
 	char seek[33];
-	char find[33];
 	const char *prefix = NULL;
 	const char *ext;
-	char *cp, *ep;
-	int fpos;
+	char *cp;
+
+	path[0] = 0;
 
 #ifdef	WIN32
+	HKEY key;
+	LONG value;
+	DWORD keynamelen, keytextlen;
+	TCHAR keyname[256];
+	TCHAR keytext[256];
+	DWORD keyindex = 0;
+	char *regprefix = getenv("CONFIG_REGISTRY");
+
+	if(!regprefix)
+		regprefix="";
+
+	cp = (char *)keypath;
+	if(*cp == '~')
+	{
+		++cp;
+		if(*cp == '/')
+			++cp;
+		snprintf(path, sizeof(path), CAPE_REGISTRY_USERSETTINGS "/%s%s/", regprefix, cp); 
+	}
+	else
+	{
+		if(*cp == '/')
+			++cp;
+		snprintf(path, sizeof(path), CAPE_REGISTRY_APPLICATIONS "/%s%s/", regprefix, cp);
+	}
+
+	cp = path;
+	while(NULL != (cp = strchr(cp, '/')))
+		*cp = '\\';
+
+	if(*keypath == '~')
+		value = RegOpenKey(HKEY_CURRENT_USER, path, &key);
+	else
+		value = RegOpenKey(HKEY_LOCAL_MACHINE, path, &key);
+
+	// if registry key path is found, then we use registry values
+	// and ignore .ini files...
+
+	if(value == ERROR_SUCCESS)
+	{
+		for(;;)
+		{
+			keynamelen = 256;
+			value = RegEnumKeyEx(key, keyindex, keyname, &keynamelen, NULL, NULL, NULL, NULL);
+			if(value != ERROR_SUCCESS)
+				break;
+
+			keytextlen = 256;
+			keytext[0] = '\0';
+			value = RegEnumValue(key, keyindex++, keytext, &keytextlen, NULL, NULL, NULL, NULL);
+			if(value != ERROR_SUCCESS)
+				continue;
+
+			if(pre)
+			{
+				snprintf(path, sizeof(path), "%s.%s", pre, keyname);
+				setValue(path, keytext);
+			}
+			else
+				setValue(keyname, keytext);
+		}
+		RegCloseKey(key);
+		return;
+	}
+
+	// windows will not support subdir .ini tree; now if not in
+	// registry, then assume unavailable
+
+	cp = strchr(keypath + 3, '/');
+	if(cp)
+		cp = strchr(cp, '/');
+
+	if(cp)
+		return;
+
 	if(*keypath == '~')
 	{
 		prefix = getenv("USERPROFILE");
@@ -295,7 +394,11 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 		setString(path, sizeof(path) - 8, prefix);
 		addString(path, sizeof(path), "\\");
 		++keypath;
+		cp = path;
+		while(NULL != (cp = strchr(cp, '\\')))
+			*cp = '/';
 	}
+
 #else
 
 	if(*keypath == '~')
@@ -313,14 +416,32 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 
 	if(!prefix)
 	{
-		prefix = getenv(environment);
+		if(!prefix && environment)
+			prefix = getenv(environment);
+
 #ifdef	WIN32
-		if(!prefix)
-			prefix = getenv("windir");
-#endif
-		if(!prefix)
+		if(!prefix || !*prefix)
+			prefix = registry.configdir;
+#else
+		if(!prefix || !*prefix)
 			prefix = ETC_PREFIX;
+#endif
+
 		setString(path, sizeof(path) - 8, prefix);
+
+#ifdef	WIN32
+		cp = path;
+		while(NULL != (cp = strchr(cp, '\\')))
+			*cp = '/';
+
+		cp = cp + strlen(path) - 1;
+		if(*cp != '/')
+		{
+			*(++cp) = '/';
+			*(++cp) = 0;
+		}
+#endif
+
 		prefix = NULL;
 	}
 
@@ -329,19 +450,10 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 
 	addString(path, sizeof(path), keypath);
 	cp = strrchr(path, '/');
-#ifdef	WIN32
-	if(!cp)
-		cp = strrchr(path, '\\');
-#endif
-
 	setString(seek, sizeof(seek), cp + 1);
 	*cp = 0;
 
 	ext = strrchr(path, '/');
-#ifdef	WIN32
-	if(!ext)
-		ext = strrchr(path, '\\');
-#endif
 	if(ext)
 		ext = strrchr(ext + 2, '.');
 	else
@@ -356,15 +468,28 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 	else if(prefix && !ext)
 		addString(path, sizeof(path), "rc");
 #endif
+	loadFile(path, seek, pre);
+}
 
+void Keydata::loadFile(const char *path, const char *keys, const char *pre)
+{
+	char seek[33];
+	char find[33];
+	char line[256];
+	char buffer[256];
+	char *cp, *ep;
+	int fpos;
+
+	if(seek)
+		setString(seek, sizeof(seek), keys);
+	else
+		seek[0] = 0;
 
 	if(strcmp(path, lastpath))
 	{
 		endKeydata();
 		if(canAccess(path))
 			cfgFile->open(path, ios::in);
-		else if(canAccess(path + 5) && !prefix)
-			cfgFile->open(path + 5, ios::in);
 		else
 			return;
 		if(!cfgFile->is_open())
@@ -382,7 +507,7 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 	cfgFile->seekg(0);
 	while(stricmp(seek, find))
 	{
-		cfgFile->getline(path, sizeof(path) - 1);
+		cfgFile->getline(line, sizeof(line) - 1);
 		if(cfgFile->eof())
 		{
 			lastpath[0] = 0;
@@ -391,7 +516,7 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 			return;
 		}
 
-		cp = path;
+		cp = line;
 		while(*cp == ' ' || *cp == '\n' || *cp == '\t')
 			++cp;
 
@@ -417,9 +542,9 @@ void Keydata::loadPrefix(const char *pre, const char *keypath, const char *envir
 			return;
 		}
 
-		cfgFile->getline(path, sizeof(path) - 1);
+		cfgFile->getline(line, sizeof(line) - 1);
 
-		cp = path;
+		cp = line;
 		while(*cp == ' ' || *cp == '\t' || *cp == '\n')
 			++cp;
 
@@ -507,95 +632,6 @@ void Keydata::end(void)
 	Keydata::cfgFile->clear();
 }
 
-// sorted by the usual probability of occurence
-// see also: manpage of isspace()
-const char * const StringTokenizer::SPACE=" \t\n\r\f\v";
-
-StringTokenizer::StringTokenizer (const char *_str,
-				  const char *_delim,
-				  bool _skipAll,
-				  bool _trim) 
-	: str(_str),delim(_delim),skipAll(_skipAll),trim(_trim) {
-	if (str == 0)
-		itEnd = iterator(*this, 0);
-	else
-		itEnd = iterator(*this,strchr(str, '\0')+1);
-}
-
-StringTokenizer::StringTokenizer (const char *s) 
-	: str(s), delim(SPACE), skipAll(false),trim(true) {
-	if (str == 0)
-		itEnd = iterator(*this, 0);
-	else
-		itEnd = iterator(*this,strchr(str, '\0')+1);
-}
-
-
-StringTokenizer::iterator& StringTokenizer::iterator::operator ++ () 
-	THROWS (StringTokenizer::NoSuchElementException) {
-
-	// someone requested to read beyond the end .. tsts
-	if (endp == myTok->itEnd.endp) 
-		THROW (NoSuchElementException());
-
-	if (token) { 
-		// this is to help people find their bugs, if they
-		// still maintain a pointer to this invalidated
-		// area :-)
-		*token = '\0'; 
-		delete[] token; 
-		token = 0;
-	}
-
-	start = ++endp;
-	if (endp == myTok->itEnd.endp) return *this; // done
-
-	// search for next delimiter
-	while (*endp && strchr(myTok->delim, *endp)==NULL)
-		++endp;
-
-	tokEnd = endp;
-	
-	if (*endp && myTok->skipAll) { // skip all delimiters
-		while (*(endp+1) && strchr(myTok->delim, *(endp+1)))
-			++endp;
-	}
-	return *this;
-}
-
-/*
- * if no one requests the token, no time is spent skipping the whitespaces
- * or allocating memory.
- */
-const char * StringTokenizer::iterator::operator * ()
-	THROWS (StringTokenizer::NoSuchElementException) {
-	// someone requested to read beyond the end .. tsts
-	if (endp == myTok->itEnd.endp)
-		THROW (NoSuchElementException());
-
-	if (!token) {
-		/*
-		 * someone requests this token; return a copy to provide
-		 * a NULL terminated string.
-		 */
-
-		/* don't clobber tokEnd, it is used in nextDelimiter() */
-		const char *wsTokEnd = tokEnd;
-		if (myTok->trim) {
-			while (wsTokEnd > start && strchr(SPACE, *start))
-				++start;
-			while (wsTokEnd > start && strchr(SPACE,*(wsTokEnd-1)))
-				--wsTokEnd;
-		}
-		size_t tokLen = wsTokEnd - start;
-		if (start > wsTokEnd) {
-			tokLen = 0;
-		}
-		token = newString(start, tokLen + 1);
-	}
-	return token;
-}
-
 #ifdef	CCXX_NAMESPACES
 }
 #endif
