@@ -139,11 +139,11 @@ void *MemPager::alloc(size_t size)
 	char *ptr;
 	struct _page *npage;
 
-	if(palign > 1)
-	{
-		size_t align = size % palign;
-		if (align) size += palign - align;
-	}
+#if __BYTE_ALIGNMENT > 1
+	size_t align = size % palign;
+	if (align) 
+		size += palign - align;
+#endif
 
 	if(page->used + size > pagesize)
 	{
@@ -173,11 +173,12 @@ void *MemPager::first(size_t size)
 	struct _page *npage = page;
 	char *ptr;
 
-	if(palign > 1)
-	{
-		size_t align = size % palign;
-		if (align) size += palign - align;
-	}
+#if	__BYTE_ALIGNMENT > 1
+	size_t align = size % palign;
+		
+	if (align) 
+		size += palign - align;
+#endif
 
 	while(npage)
 	{
@@ -209,6 +210,50 @@ char *MemPager::first(char *str)
 	return setString(cp, len, str);
 }
 
+StackPager::StackPager(size_t pg) :
+MemPager(pg)
+{
+	stack = NULL;
+}
+
+void StackPager::purge(void)
+{
+	MemPager::purge();
+	stack = NULL;
+}
+
+void *StackPager::pull(void)
+{
+	frame_t *object = stack;
+
+	if(!stack)
+	{
+		purge();
+		return NULL;
+	}
+
+	stack = object->next;
+	return object->data;	
+}
+
+void *StackPager::push(const void* object, size_t len)
+{
+	frame_t *frame = (frame_t *)alloc(len + sizeof(frame_t) - 1);
+
+	if(!frame)
+		return NULL;
+	
+	frame->next = stack;
+	stack = frame;
+	memcpy(frame->data, object, len);
+	return (void *)frame->data;
+}
+
+void *StackPager::push(const char *string)
+{
+	return push(string, strlen(string) + 1);
+}
+
 SharedMemPager::SharedMemPager(size_t pg, const char *name) :
 MemPager(pg), Mutex(name)
 {}
