@@ -356,22 +356,8 @@ Mutex::~Mutex()
 
 }
 	
-
-#endif
-
-#ifndef PTHREAD_MUTEXTYPE_RECURSIVE
 void Mutex::leaveMutex(void)
 {
-#ifndef WIN32
-	if(_level > 1)
-	{
-		--_level;
-		return;
-	}
-	--_level;
-	_tid = NULL;
-	pthread_mutex_unlock(&_mutex);
-#else // WIN32
 #ifdef MUTEX_UNDERGROUND_WIN32_CRITICALSECTION 
 	::LeaveCriticalSection(&_criticalSection);
 #endif // MUTEX_UNDERGROUND_WIN32_CRITICALSECTION
@@ -380,10 +366,8 @@ void Mutex::leaveMutex(void)
 	if (!ReleaseMutex(_mutex))
 		THROW(this);
 #endif // MUTEX_UNDERGROUND_WIN32_MUTEX
-
-#endif // !WIN32
 }
-#endif
+#endif	// WIN32 MUTEX
 
 #ifdef WIN32
 MutexCounter::MutexCounter(const char *id) : Mutex(id)
