@@ -135,7 +135,7 @@ bool enumNetworkDevices(vector<NetworkDeviceInfo>& devs)
 		else
 			(IPV4Address&)maskaddr = ((sockaddr_in&)devifreq.ifr_addr).sin_addr;
 
-#ifdef SIOCGLIFMTU
+#if defined(SIOCGLIFMTU) && !defined(__hpux)
 		struct lifreq devlifreq;
 		if(ioctl(fd, SIOCGLIFMTU, &devlifreq) == -1)
 			mtu = 0;
@@ -145,7 +145,11 @@ bool enumNetworkDevices(vector<NetworkDeviceInfo>& devs)
 		if(ioctl(fd, SIOCGIFMTU, &devifreq) == -1)
 			mtu = 0;
 		else
+#if defined(__hpux)
+			mtu = devifreq.ifr_metric;
+#else
 			mtu = devifreq.ifr_mtu;
+#endif
 #endif
 		devs.push_back(NetworkDeviceInfo(ifc.ifc_req[i].ifr_name, addr, brdaddr, maskaddr, mtu));
 	}
