@@ -63,6 +63,7 @@ UDPSocket(ia, port)
 {
 	disconnect();	// assure not started live
 	shutdown(so, 0);
+	receiveBuffer(0);
 }
 
 #ifdef	CCXX_IPV6
@@ -71,6 +72,7 @@ UDPSocket(ia, port)
 {
         disconnect();   // assure not started live
         shutdown(so, 0);
+	receiveBuffer(0);
 }
 #endif
 
@@ -78,6 +80,7 @@ UDPTransmit::UDPTransmit(Family family) : UDPSocket(family)
 {
 	disconnect();
 	shutdown(so, 0);
+	receiveBuffer(0);
 }
 
 Socket::Error UDPTransmit::cConnect(const IPV4Address &ia, tpport_t port)
@@ -95,7 +98,9 @@ Socket::Error UDPTransmit::cConnect(const IPV4Address &ia, tpport_t port)
 	return errSuccess;
 }
 
+
 #ifdef	CCXX_IPV6
+
 Socket::Error UDPTransmit::connect(const IPV6Address &ia, tpport_t port)
 {
         int len = sizeof(peer.ipv6);
@@ -105,8 +110,9 @@ Socket::Error UDPTransmit::connect(const IPV6Address &ia, tpport_t port)
         peer.ipv6.sin6_port = htons(port);
         // Win32 will crash if you try to connect to INADDR_ANY.
 	if(!memcmp(&peer.ipv6.sin6_addr, &in6addr_any, sizeof(in6addr_any)))
-		memcpy(&peer.ipv6.sin6_addr, &in6addr_loopback, sizeof(in6addr_loopback));
-        if(::connect(so, (sockaddr *)&peer.ipv6, len))
+		memcpy(&peer.ipv6.sin6_addr, &in6addr_loopback, 
+			sizeof(in6addr_loopback));
+        if(::connect(so, (struct sockaddr *)&peer.ipv6, len))
                 return connectError();
         return errSuccess;
 }
@@ -152,6 +158,7 @@ UDPReceive::UDPReceive(const IPV4Address &ia, tpport_t port) :
 UDPSocket(ia, port)
 {
 	shutdown(so, 1);
+	sendBuffer(0);
 }
 
 #ifdef	CCXX_IPV6
@@ -159,6 +166,7 @@ UDPReceive::UDPReceive(const IPV6Address &ia, tpport_t port) :
 UDPSocket(ia, port)
 {
         shutdown(so, 1);
+	sendBuffer(0);
 }
 #endif
 
@@ -172,7 +180,7 @@ Socket::Error UDPReceive::connect(const IPV4Host &ia, tpport_t port)
 	// Win32 will crash if you try to connect to INADDR_ANY.
 	if ( INADDR_ANY == peer.ipv4.sin_addr.s_addr )
 	        peer.ipv4.sin_addr.s_addr = INADDR_LOOPBACK;
-	if(::connect(so, (sockaddr *)&peer.ipv4, len))
+	if(::connect(so, (struct sockaddr *)&peer.ipv4, len))
 		return connectError();
 	return errSuccess;
 }
