@@ -50,169 +50,85 @@ namespace ost {
 #endif
 
 #ifndef WIN32
-#ifdef	CCXX_SYSV_SEMAPHORES
 
 extern "C" {
 #include <sys/types.h>
-#include <sys/ipc.h>
-#include <sys/sem.h>
-};
+}
 
 #include <cerrno>
 
-Semaphore::Semaphore(size_t resource)
+Semaphore::Semaphore(unsigned resource)
 {
-#ifdef _AIX
-        union semun
-        {
-                int val; 
-                struct semid_ds *buf; 
-                unsigned short *array; 
-        } arg;
-#else
-	union semun arg;
-#endif
+	pthread_mutexattr_t attr;
+        pthread_mutexattr_init(&attr);
+        pthread_mutex_init(&_mutex, &attr);
+        pthread_mutexattr_destroy(&attr);
+        if(pthread_cond_init(&_cond, NULL) && Thread::getException() == Thread::throwObject)
+                THROW(this);
 
-	if((_semaphore = semget(IPC_PRIVATE, 1, 0644 | IPC_CREAT)) == -1)
-	{
-#ifdef	CCXX_EXCEPTIONS
-		if(Thread::getException() == Thread::throwObject)
-			throw(this);
-#ifdef	COMMON_STD_EXCEPTION
-		else if(Thread::getException() == Thread::throwException)
-			throw(SyncException("semaphone create failed"));
-#endif
-#endif
-		return;
-	}
-
-	arg.val = resource;
-	if(semctl(_semaphore, 0, SETVAL, arg) == -1)
-	{
-#ifdef	CCXX_EXCEPTIONS
-		if(Thread::getException() == Thread::throwObject)
-			throw(this);
-#ifdef	COMMON_STD_EXCEPTION
-		else if(Thread::getException() == Thread::throwException)
-			throw(SyncException("semaphore control failed"));
-#endif
-#endif
-		return;
-	}
+	_count = resource;
+	_waiters = 0;
 }
 
 Semaphore::~Semaphore()
 {
-	semctl(_semaphore, 0, IPC_RMID);
+	pthread_cond_destroy(&_cond);
+        pthread_mutex_destroy(&_mutex);
 }
 
-void Semaphore::wait(void)
+bool Semaphore::wait(timeout_t timeout)
 {
-	struct sembuf ops[] = {{0, -1, 0}};
-	
-	semop(_semaphore, ops, 1);
-}
+	struct timespec ts;
+	bool flag = true;
+	int rc;
+	pthread_mutex_lock(&_mutex);
+	++_waiters;
 
-bool Semaphore::tryWait(void)
-{
-	struct sembuf ops[] = {{0, -1, IPC_NOWAIT}};
-	
-	return (semop(_semaphore, ops, 1) == EAGAIN) ? false : true;
-}
+	if(_count)
+		goto exiting;
 
-void Semaphore::post(void)
-{
-	struct sembuf ops[] = {{0, 1, 0}};
-
-	semop(_semaphore, ops, 1);
-}
-
-int Semaphore::getValue(void)
-{
-	return semctl(_semaphore,0,GETVAL);
-}
-
-#else // CCXX_SYSV_SEMAPHORES
-
-Semaphore::Semaphore(size_t resource) 
-{
-#ifdef	CCXX_NAMED_SEMAPHORES
-	static int semnum = 0;
-	char name[33];
-
-	snprintf(name, sizeof(name), "/tmp/.sem-%d-%4.4d", getpid(), semnum++);
-	_semobject = sem_open(name, O_CREAT, 0600, resource);
-	if(_semobject == (sem_t *)SEM_FAILED)
-		_semobject = NULL;
-	if(_semobject)
-		sem_unlink(name);
-	else
+	if(!timeout)
 	{
-#else
-	_semobject = &_semaphore;
-	if(sem_init(&_semaphore, 0, resource))
-	{
-		_semobject = NULL;
-#endif
-#ifdef	CCXX_EXCEPTIONS
-		if(Thread::getException() == Thread::throwObject)
-			throw(this);
-#ifdef	COMMON_STD_EXCEPTION
-		else if(Thread::getException() == Thread::throwException)
-			throw(SyncException("semaphore failed"));
-#endif
-#endif
+		while(_count == 0)
+			pthread_cond_wait(&_cond, &_mutex);
+		goto exiting;
 	}
-}
 
-Semaphore::~Semaphore()
-{
-#ifdef	CCXX_NAMED_SEMAPHORES
-	if(_semobject)
-		sem_close(_semobject);
-#else
-	if(_semobject)
-		sem_destroy(&_semaphore);
-#endif
-}
+	getTimeout(&ts, timeout);
+	rc = pthread_cond_timedwait(&_cond, &_mutex, &ts);
+	if(rc == ETIMEDOUT || _count == 0)
+		flag = false;
 
-void Semaphore::wait(void)
-{
-	if(_semobject)
-		sem_wait(_semobject);
+exiting:
+	--_waiters;
+	if(_count)
+		--_count;
+	pthread_mutex_unlock(&_mutex);
+	return flag;
 }
 
 void Semaphore::post(void)
 {
-	if(_semobject)
-		sem_post(_semobject);
+	pthread_mutex_lock(&_mutex);
+	if(_waiters > 0)
+		pthread_cond_signal(&_cond);
+	++_count;
+	pthread_mutex_unlock(&_mutex);
 }
 
-bool Semaphore::tryWait(void)
-{
-	if(!_semobject)
-		return false;
-	return (sem_trywait(_semobject) == 0) ? true : false;
-}
-
-#ifndef __CYGWIN32__
 int Semaphore::getValue(void)
 {
 	int value;
 
-	if(!_semobject)
-		return 0;
-
-	sem_getvalue(_semobject, &value);
+	pthread_mutex_lock(&_mutex);
+	value = _count;
+	pthread_mutex_unlock(&_mutex);
 	return value;
 }
-#endif
-
-#endif //CCXX_SYSV_SEMAPHORES
 
 #else
 
-Semaphore::Semaphore(size_t resource)
+Semaphore::Semaphore(unsigned resource)
 {
 	semObject = ::CreateSemaphore((LPSECURITY_ATTRIBUTES)NULL, (LONG)resource, MAX_SEM_VALUE, (LPCTSTR)NULL);
 }	
@@ -222,14 +138,12 @@ Semaphore::~Semaphore()
 	::CloseHandle(semObject);
 }
 
-void Semaphore::wait(void)
+bool Semaphore::wait(timeout_t timeout)
 {
-	Thread::waitThread(semObject, INFINITE);
-}
+	if(!timeout)
+		timeout = INFINITE;
 
-bool Semaphore::tryWait(void)
-{
-	return Thread::waitThread(semObject, 0) == WAIT_OBJECT_0;
+	return Thread::waitThread(semObject, timeout) == WAIT_OBJECT_0;
 }
 
 void Semaphore::post(void)
