@@ -61,11 +61,24 @@
 #define socket_errno	WSAGetLastError()
 #endif
 
-#ifdef WIN32
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#else
+#ifndef WIN32
 #include <sys/ioctl.h>
+#ifdef HAVE_NET_IP6_H
+#include <netinet/ip6.h>
+#endif
+
+#if defined(__hpux) && defined(_XOPEN_SOURCE_EXTENDED)
+#undef	_XOPEN_SOURCE_EXTENDED
+#endif
+
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+
+#if defined(__hpux)
+#define	_XOPEN_SOURCE_EXTENDED
+#endif
+
 #ifdef	HAVE_NET_IF_H
 #include <net/if.h>
 #endif
@@ -109,6 +122,40 @@ static SOCKET dupSocket(SOCKET so,enum Socket::State state)
 # define DUP_SOCK(s,state) dup(s)
 #endif 
 
+Mutex Socket::mutex;
+
+bool Socket::check(Family fam)
+{
+	SOCKET so = INVALID_SOCKET;
+
+	switch(fam)
+	{
+	case IPV4:
+		so = socket(fam, SOCK_DGRAM, 0);
+		break;
+#ifdef	CCXX_IPV6
+	case IPV6:
+		so = socket(fam, SOCK_DGRAM, 0);
+		break;
+#endif
+	}
+
+	if(so == INVALID_SOCKET)
+		return false;
+
+#ifdef	WIN32
+	closesocket(so);
+#else
+	close(so);
+#endif
+	return true;
+}
+
+Socket::Socket()
+{
+	setSocket();
+}
+
 Socket::Socket(int domain, int type, int protocol)
 {
 	setSocket();
@@ -118,6 +165,10 @@ Socket::Socket(int domain, int type, int protocol)
 		error(errCreateFailed,"Could not create socket",socket_errno);
 		return;
 	}
+#ifdef	SO_NOSIGPIPE
+	int opt = 1;
+	setsockopt(so, SOL_SOCKET, SO_NOSIGPIPE, (char *)&opt, sizeof(opt));
+#endif
 	state = AVAILABLE;
 }
 
@@ -164,6 +215,98 @@ void Socket::setSocket(void)
 	so              = INVALID_SOCKET;
 }
 
+Socket::Error Socket::sendLimit(int limit)
+{
+#ifdef	SO_SNDLOWAT
+	if(setsockopt(so, SOL_SOCKET, SO_SNDLOWAT, (char *)&limit, sizeof(limit)))
+		return errInvalidValue;
+
+	return errSuccess;
+	
+#else
+	return errServiceUnavailable;
+#endif
+}
+
+Socket::Error Socket::receiveLimit(int limit)
+{
+#ifdef  SO_RCVLOWAT
+        if(setsockopt(so, SOL_SOCKET, SO_RCVLOWAT, (char *)&limit, sizeof(limit)))
+                return errInvalidValue;
+
+        return errSuccess;
+
+#else
+        return errServiceUnavailable;
+#endif
+}
+
+Socket::Error Socket::sendTimeout(timeout_t to)
+{
+#ifdef  SO_SNDTIMEO
+        struct timeval tv;
+
+        tv.tv_sec = to / 1000;
+        tv.tv_usec = (to % 1000) * 1000;
+
+        if(setsockopt(so, SOL_SOCKET, SO_SNDTIMEO, (char *)&tv, sizeof(tv)))
+                return errInvalidValue;
+
+        return errSuccess;
+
+#else
+        return errServiceUnavailable;
+#endif
+}
+
+Socket::Error Socket::receiveTimeout(timeout_t to)
+{
+#ifdef  SO_RCVTIMEO
+        struct timeval tv;
+
+        tv.tv_sec = to / 1000;
+        tv.tv_usec = (to % 1000) * 1000;
+
+        if(setsockopt(so, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv, sizeof(tv)))
+                return errInvalidValue;
+
+        return errSuccess;
+#else
+        return errServiceUnavailable;
+#endif
+}	
+
+Socket::Error Socket::sendBuffer(unsigned bufsize)
+{
+#ifdef	SO_SNDBUF
+	if(setsockopt(so, SOL_SOCKET, SO_SNDBUF, (char *)&bufsize, sizeof(bufsize)))
+		return errInvalidValue;
+	return errSuccess;
+#else
+	return errServiceUnavailable;
+#endif
+}
+		
+Socket::Error Socket::bufferSize(unsigned bufsize)
+{
+	Error err = receiveBuffer(bufsize);
+	if(err == errSuccess)
+		err = sendBuffer(bufsize);
+
+	return err;
+}
+
+Socket::Error Socket::receiveBuffer(unsigned bufsize)
+{
+#ifdef  SO_RCVBUF
+        if(setsockopt(so, SOL_SOCKET, SO_RCVBUF, (char *)&bufsize, sizeof(bufsize)))
+		return errInvalidValue;
+        return errSuccess;
+#else
+        return errServiceUnavailable; 
+#endif
+}
+
 ssize_t Socket::readLine(char *str, size_t request, timeout_t timeout)
 {
 	bool crlf = false;
@@ -186,7 +329,7 @@ ssize_t Socket::readLine(char *str, size_t request, timeout_t timeout)
 				return -1;
 			}
 		}
-		nstat = ::recv(so, str, IOLEN nleft, MSG_PEEK);
+		nstat = ::recv(so, str, _IOLEN64 nleft, MSG_PEEK);
 		if(nstat <= 0)
 		{
 			error(errInput,"Could not read from socket", socket_errno);
@@ -246,7 +389,7 @@ ssize_t Socket::readData(void *Target, size_t Size, char Separator, timeout_t ti
             error(errTimeout);
             return (-1);
           }
-      nstat =::recv (so, (char *)Target, IOLEN Size, 0);
+      nstat =::recv (so, (char *)Target, _IOLEN64 Size, 0);
 
       if (nstat < 0)
         {
@@ -274,7 +417,7 @@ ssize_t Socket::readData(void *Target, size_t Size, char Separator, timeout_t ti
             return (-1);
           }
 
-      nstat =::recv (so, str, IOLEN nleft, MSG_PEEK);
+      nstat =::recv (so, str, _IOLEN64 nleft, MSG_PEEK);
       if (nstat <= 0)
         {
           error (errInput);
@@ -298,7 +441,7 @@ ssize_t Socket::readData(void *Target, size_t Size, char Separator, timeout_t ti
 
 ssize_t Socket::writeData(const void *Source, size_t Size, timeout_t timeout)
 {
-  if (Size < 0)
+  if (Size < 1)
     return (0);
 
   ssize_t nstat;
@@ -313,7 +456,7 @@ ssize_t Socket::writeData(const void *Source, size_t Size, timeout_t timeout)
             return (-1);
           }
 
-      nstat =::send (so, Slide, IOLEN Size, 0);
+      nstat =::send (so, Slide, _IOLEN64 Size, 0);
 
       if (nstat <= 0)
         {
@@ -354,7 +497,7 @@ void Socket::endSocket(void)
 		{
 			SOCKET sosave = so;
 			so = INVALID_SOCKET;
-			close((int)sosave);
+			closesocket((int)sosave);
 		}
 #else
 		if(so > -1)
@@ -707,13 +850,13 @@ bool Socket::isPending(Pending pending, timeout_t timeout)
 	switch(pending)
 	{
 	case pendingInput:
-		status = select(so + (SOCKET)1, &grp, NULL, NULL, tvp);
+		status = select((int)so + 1, &grp, NULL, NULL, tvp);
 		break;
 	case pendingOutput:
-		status = select(so + (SOCKET)1, NULL, &grp, NULL, tvp);
+		status = select((int)so + 1, NULL, &grp, NULL, tvp);
 		break;
 	case pendingError:
-		status = select(so + (SOCKET)1, NULL, NULL, &grp, tvp);
+		status = select((int)so + 1, NULL, NULL, &grp, tvp);
 		break;
 	}
 #ifdef	WIN32
@@ -907,7 +1050,7 @@ IPV4Host Socket::getIPV4NAT(tpport_t *port) const
 	socklen_t len = sizeof(addr);
 	natResult res;
 
-	if((res=natv4Lookup(so, &addr))!=natOK)
+	if((res=natv4Lookup((int)so, &addr))!=natOK)
 	{
 	  	if(res==natNotSupported)
 		  	error( errServiceUnavailable, natErrorString( res ) );
@@ -1137,6 +1280,181 @@ Socket::Error Socket::drop(const IPV6Multicast &ia)
 }
 #endif
 
+#ifdef	HAVE_GETADDRINFO
+
+UDPSocket::UDPSocket(const char *name, Family fam) :
+Socket(fam, SOCK_DGRAM, 0)
+{
+	char namebuf[128], *cp;
+	struct addrinfo hint, *list = NULL, *first;
+
+	family = fam;
+
+	switch(fam)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		peer.ipv6.sin6_family = family;
+		break;
+#endif
+	default:
+		peer.ipv4.sin_family = family;
+	}
+	
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+	cp = strrchr(namebuf, '/');
+	if(!cp && family == IPV4)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp)
+	{
+		cp = namebuf;
+		name = NULL;
+	}
+	else
+	{
+		name = namebuf;
+		*(cp++) = 0;
+		if(!strcmp(name, "*"))
+			name = NULL;
+	}
+
+	memset(&hint, 0, sizeof(hint));
+
+	hint.ai_family = family;
+	hint.ai_socktype = SOCK_DGRAM;
+	hint.ai_protocol = IPPROTO_UDP;
+	hint.ai_flags = AI_PASSIVE;
+
+	if(getaddrinfo(name, cp, &hint, &list) || !list)
+	{
+		error(errBindingFailed, "Could not find service", errno);
+		endSocket();
+		return;
+	}
+
+#if defined(SO_REUSEADDR) && !defined(WIN32)
+        int opt = 1;
+        setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt,
+                (socklen_t)sizeof(opt));
+#endif	
+
+	first = list;
+
+	while(list)
+	{
+		if(!bind(so, list->ai_addr, (socklen_t)list->ai_addrlen))
+		{
+			state = BOUND;
+			break;
+		}
+		list = list->ai_next;
+	}
+	freeaddrinfo(first);
+
+	if(state != BOUND)
+	{
+		endSocket();
+		error(errBindingFailed, "Count not bind socket", errno);
+		return;
+	}
+}
+
+#else
+
+UDPSocket::UDPSocket(const char *name, Family fam) :
+Socket(fam, SOCK_DGRAM, 0)
+{
+	char namebuf[128], *cp;
+	tpport_t port;
+	struct servent *svc;
+	socklen_t alen;
+	struct sockaddr *addr;
+
+	family = fam;
+	
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+	cp = strrchr(namebuf, '/');
+	if(!cp && family == IPV4)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp)
+	{
+		cp = namebuf;
+		name = "*";
+	}
+	else
+	{
+		name = namebuf;
+		*(cp++) = 0;
+	}
+	
+	if(isdigit(*cp))
+		port = atoi(cp);
+	else
+	{
+		mutex.enter();
+		svc = getservbyname(cp, "udp");
+		if(svc)
+			port = ntohs(svc->s_port);
+		mutex.leave();
+		if(!svc)
+		{
+			error(errBindingFailed, "Could not find service", errno);
+			endSocket();
+			return;
+		}
+	}
+
+	struct sockaddr_in addr4;
+	IPV4Address ia4(name);
+#ifdef	CCXX_IPV6
+	struct sockaddr_in6 addr6;
+	IPV6Address ia6(name);
+#endif
+	
+	switch(family)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		peer.ipv6.sin6_family = family;
+		memset(&addr6, 0, sizeof(addr6));
+		addr6.sin6_family = family;
+		addr6.sin6_addr = getaddress(ia6);
+		addr6.sin6_port = htons(port);
+		alen = sizeof(addr6);
+		addr = (struct sockaddr *)&addr6;
+		break;
+#endif
+	case IPV4:
+		peer.ipv4.sin_family = family;
+		memset(&addr4, 0, sizeof(addr4));
+		addr4.sin_family = family;
+		addr4.sin_addr = getaddress(ia4);
+		addr4.sin_port = htons(port);
+		alen = sizeof(&addr4);
+		addr = (struct sockaddr *)&addr4;
+	}
+
+#if defined(SO_REUSEADDR) && !defined(WIN32)
+        int opt = 1;
+        setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt,
+                (socklen_t)sizeof(opt));
+#endif	
+
+	if(!bind(so, addr, alen))
+		state = BOUND;
+
+	if(state != BOUND)
+	{
+		endSocket();
+		error(errBindingFailed, "Count not bind socket", errno);
+		return;
+	}
+}
+
+#endif
+
 UDPSocket::UDPSocket(Family fam) :
 Socket(fam, SOCK_DGRAM, 0)
 {
@@ -1203,10 +1521,64 @@ UDPSocket::~UDPSocket()
 	endSocket();
 }
 
+ssize_t UDPSocket::send(const void *buf, size_t len)
+{
+	struct sockaddr *addr;
+	socklen_t alen;
+
+	switch(family)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		addr = (struct sockaddr *)&peer.ipv6;
+		alen = sizeof(struct sockaddr_in6);
+		break;
+#endif
+	case IPV4:
+		addr = (struct sockaddr *)&peer.ipv4;
+		alen = sizeof(struct sockaddr_in);
+	}
+
+	if(isConnected())
+	{
+		addr = NULL;
+		alen = 0;
+	}
+	
+	return _IORET64 ::sendto(so, (const char *)buf, _IOLEN64 len, 0, addr, alen);
+}
+
+ssize_t UDPSocket::receive(void *buf, size_t len, bool reply)
+{
+	struct sockaddr *addr;
+	socklen_t alen;
+
+	switch(family)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		addr = (struct sockaddr *)&peer.ipv6;
+		alen = sizeof(struct sockaddr_in6);
+		break;
+#endif
+	case IPV4:
+		addr = (struct sockaddr *)&peer.ipv4;
+		alen = sizeof(struct sockaddr_in);
+	}
+
+	if(isConnected() || !reply)
+	{
+		addr = NULL;
+		alen = 0;
+	}
+	
+	return _IORET64 ::recvfrom(so, (char *)buf, _IOLEN64 len, 0, addr, &alen);
+}
+
 Socket::Error UDPSocket::join(const IPV4Multicast &ia,int InterfaceIndex)
 {
 #ifndef WIN32
-#if defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__)
+#if defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__) && !defined(_OSF_SOURCE) && !defined(__hpux)
 
         struct ip_mreqn      group;
         struct sockaddr_in   myaddr;
@@ -1233,7 +1605,7 @@ Socket::Error UDPSocket::join(const IPV4Multicast &ia,int InterfaceIndex)
 Socket::Error UDPSocket::getInterfaceIndex(const char *DeviceName,int& InterfaceIndex)
 {
 #ifndef WIN32
-#if defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__)
+#if defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__) && !defined(_OSF_SOURCE) && !defined(__hpux)
 
   struct ip_mreqn  mreqn;
   struct ifreq     m_ifreq;
@@ -1271,6 +1643,12 @@ Socket::Error UDPSocket::disconnect(void)
 {
 	struct sockaddr_in addr;
         int len = sizeof(addr);
+
+	if(so == INVALID_SOCKET)
+		return errSuccess;
+
+	Socket::state = BOUND;
+
         memset(&addr, 0, len);
 #ifndef WIN32
         addr.sin_family = AF_UNSPEC;
@@ -1285,6 +1663,10 @@ Socket::Error UDPSocket::disconnect(void)
 #else   
 Socket::Error UDPSocket::disconnect(void)
 {
+	if(so == INVALID_SOCKET)
+		return errSuccess;
+
+	Socket::state = BOUND;
         return connect(getLocal());
 }
 #endif
@@ -1297,15 +1679,146 @@ void UDPSocket::setPeer(const IPV4Host &ia, tpport_t port)
 	peer.ipv4.sin_port = htons(port);
 }
 
+void UDPSocket::connect(const IPV4Host &ia, tpport_t port)
+{
+	setPeer(ia, port);
+	if(so == INVALID_SOCKET)
+		return;
+
+	if(!::connect(so, (struct sockaddr *)&peer.ipv4, sizeof(struct sockaddr_in)))
+		Socket::state = CONNECTED;
+}
+
 #ifdef	CCXX_IPV6
 void UDPSocket::setPeer(const IPV6Host &ia, tpport_t port)
-{        memset(&peer.ipv6, 0, sizeof(peer.ipv6));
+{        
+	memset(&peer.ipv6, 0, sizeof(peer.ipv6));
         peer.ipv6.sin6_family = AF_INET6;
         peer.ipv6.sin6_addr = getaddress(ia);
         peer.ipv6.sin6_port = htons(port);
 }
+
+void UDPSocket::connect(const IPV6Host &ia, tpport_t port)
+{
+	setPeer(ia, port);
+
+	if(so == INVALID_SOCKET)
+		return;
+
+	if(!::connect(so, (struct sockaddr *)&peer.ipv6, sizeof(struct sockaddr_in6)))
+		Socket::state = CONNECTED;
+
+}
+
 #endif
 
+#ifdef  HAVE_GETADDRINFO
+
+void UDPSocket::setPeer(const char *name)
+{
+	char namebuf[128], *cp;
+	struct addrinfo hint, *list;
+
+        snprintf(namebuf, sizeof(namebuf), "%s", name);
+        cp = strrchr(namebuf, '/');
+        if(!cp)
+                cp = strrchr(namebuf, ':');
+
+	if(!cp)
+		return;
+
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = family;
+	hint.ai_socktype = SOCK_DGRAM;
+	hint.ai_protocol = IPPROTO_UDP;
+
+	if(getaddrinfo(namebuf, cp, &hint, &list) || !list)
+		return;
+
+	switch(family)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		memcpy(&peer.ipv6, list->ai_addr, sizeof(peer.ipv6));
+		break;
+#endif
+	case IPV4:
+                memcpy(&peer.ipv4, list->ai_addr, sizeof(peer.ipv4));
+	}
+
+	freeaddrinfo(list);
+}
+
+#else
+
+void UDPSocket::setPeer(const char *name)
+{
+	char namebuf[128], *cp;
+	struct servent *svc;
+	tpport_t port;
+	struct sockaddr *addr;
+	socklen_t alen;
+
+        snprintf(namebuf, sizeof(namebuf), "%s", name);
+        cp = strrchr(namebuf, '/');
+        if(!cp)
+                cp = strrchr(namebuf, ':');
+
+	if(!cp)
+		return;
+		
+	if(isdigit(*cp))
+		port = atoi(cp);
+	else
+	{
+		mutex.enter();
+		svc = getservbyname(cp, "udp");
+		if(svc)
+			port = ntohs(svc->s_port);
+		mutex.leave();
+		if(!svc)
+			return;
+	}
+	
+	memset(&peer, 0, sizeof(peer));
+	
+	switch(family)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		setPeer(IPV6Host(namebuf), port);
+		break;
+#endif
+	case IPV4:
+		setPeer(IPV4Host(namebuf), port);
+	}	
+}
+
+#endif
+
+void UDPSocket::connect(const char *service)
+{
+	int rtn;
+
+	setPeer(service);
+
+	if(so == INVALID_SOCKET)
+		return;
+
+	switch(family)
+	{
+#ifdef	CCXX_IPV6
+	case IPV6:
+		rtn = ::connect(so, (struct sockaddr *)&peer.ipv6, sizeof(struct sockaddr_in6));
+		break;
+#endif
+	case IPV4:
+		rtn = ::connect(so, (struct sockaddr *)&peer.ipv4, sizeof(struct sockaddr_in));
+	}
+	if(!rtn)
+		Socket::state = CONNECTED;
+}
+
 IPV4Host UDPSocket::getIPV4Peer(tpport_t *port) const
 {
 	// FIXME: insufficient buffer
@@ -1389,7 +1902,159 @@ void UDPBroadcast::setPeer(const IPV4Broadcast &ia, tpport_t port)
 	peer.ipv4.sin_port = htons(port);
 }
 
-TCPSocket::TCPSocket(const IPV4Address &ia, tpport_t port, int backlog) :
+void TCPSocket::setSegmentSize(unsigned mss)
+{
+#ifdef	TCP_MAXSEG
+	if(mss > 1)
+		setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, sizeof(mss));
+#endif
+	segsize = mss;
+}
+
+#ifdef	HAVE_GETADDRINFO
+TCPSocket::TCPSocket(const char *name, unsigned backlog, unsigned mss) :
+Socket(AF_INET, SOCK_STREAM, 0)
+{
+	char namebuf[128], *cp;
+	struct addrinfo hint, *list = NULL, *first;
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+        cp = strrchr(namebuf, '/');
+        if(!cp)
+                cp = strrchr(namebuf, ':');
+
+	if(!cp)
+	{
+		cp = namebuf;
+		name = NULL;
+	}
+	else
+	{
+		name = namebuf;
+		*(cp++) = 0;
+		if(!strcmp(name, "*"))
+			name = NULL;
+	}
+	
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = AF_INET;
+	hint.ai_socktype = SOCK_STREAM;
+	hint.ai_protocol = IPPROTO_TCP;
+	hint.ai_flags = AI_PASSIVE;	
+
+	if(getaddrinfo(name, cp, &hint, &list) || !list)
+	{
+		endSocket();
+		error(errBindingFailed, "Could not find service", errno);
+		return;
+	}
+
+#if defined(SO_REUSEADDR) && !defined(WIN32)
+        int opt = 1;
+        setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, 
+                (socklen_t)sizeof(opt));
+#endif
+
+	first = list;
+	while(list)
+	{
+		if(!bind(so, list->ai_addr, (socklen_t)list->ai_addrlen))
+		{
+			state = BOUND;
+			break;
+		}
+		list = list->ai_next;
+	}
+	freeaddrinfo(first);
+	if(state != BOUND)
+	{
+		endSocket();
+                error(errBindingFailed,"Could not bind socket",socket_errno);
+                return;
+	}
+
+	setSegmentSize(mss);
+        if(listen(so, backlog))
+        {
+                endSocket();
+                error(errBindingFailed,"Could not listen on socket",socket_errno);
+                return;
+        }
+}
+
+#else
+TCPSocket::TCPSocket(const char *name, unsigned backlog, unsigned mss) :
+Socket(AF_INET, SOCK_STREAM, 0)
+{
+	char namebuf[128], *cp;
+        struct sockaddr_in addr;
+	struct servent *svc;
+
+	memset(&addr, 0, sizeof(addr));
+        snprintf(namebuf, sizeof(namebuf), "%s", name);
+        cp = strrchr(namebuf, '/');
+        if(!cp)
+                cp = strrchr(namebuf, ':');
+
+        if(!cp)
+        {
+                cp = namebuf;
+                name = "*";
+        }
+        else
+        {
+                name = namebuf;
+                *(cp++) = 0;
+        }
+
+	addr.sin_family = AF_INET;
+	if(isdigit(*cp))
+		addr.sin_port = htons(atoi(cp));
+	else
+	{
+		mutex.enter();
+		svc = getservbyname(cp, "tcp");
+		if(svc)
+			addr.sin_port = svc->s_port;
+		mutex.leave();
+		if(!svc)
+		{
+                	endSocket();
+                	error(errBindingFailed, "Could not find service", errno);
+                	return;
+
+		}
+	}
+
+	IPV4Address ia(name);
+	addr.sin_addr = getaddress(ia);
+
+#if defined(SO_REUSEADDR) && !defined(WIN32)
+        int opt = 1;    
+        setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, 
+		(socklen_t)sizeof(opt));
+#endif      
+
+        if(bind(so, (struct sockaddr *)&addr, sizeof(addr)))
+        {
+                endSocket();
+                error(errBindingFailed,"Could not bind socket",socket_errno);
+                return;
+        }
+
+	setSegmentSize(mss);
+        if(listen(so, backlog)) 
+        {
+                endSocket();
+                error(errBindingFailed,"Could not listen on socket",
+			socket_errno);
+                return;
+        }
+        state = BOUND;
+}
+
+#endif	
+
+TCPSocket::TCPSocket(const IPV4Address &ia, tpport_t port, unsigned backlog, unsigned mss) :
 Socket(AF_INET, SOCK_STREAM, 0)
 {
 	struct sockaddr_in addr;
@@ -1410,6 +2075,8 @@ Socket(AF_INET, SOCK_STREAM, 0)
 		return;
 	}
 
+	setSegmentSize(mss);
+
 	if(listen(so, backlog))
 	{
 		endSocket();
@@ -1420,8 +2087,161 @@ Socket(AF_INET, SOCK_STREAM, 0)
 }
 
 #ifdef  CCXX_IPV6
-TCPV6Socket::TCPV6Socket(const IPV6Address &ia, tpport_t port, int backlog) :
-Socket(AF_INET, SOCK_STREAM, 0)
+
+void TCPV6Socket::setSegmentSize(unsigned mss)
+{
+#ifdef	TCP_MAXSEG
+	if(mss > 1)
+		setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, sizeof(mss));
+#endif
+	segsize = mss;
+}
+
+
+#ifdef	HAVE_GETADDRINFO
+TCPV6Socket::TCPV6Socket(const char *name, unsigned backlog, unsigned mss) :
+Socket(AF_INET6, SOCK_STREAM, 0)
+{
+	char namebuf[128], *cp;
+	struct addrinfo hint, *list = NULL, *first;
+
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+        cp = strrchr(namebuf, '/');
+
+	if(!cp)
+	{
+		cp = namebuf;
+		name = NULL;
+	}
+	else
+	{
+		name = namebuf;
+		*(cp++) = 0;
+		if(!strcmp(name, "*"))
+			name = NULL;
+	}
+	
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = AF_INET6;
+	hint.ai_socktype = SOCK_STREAM;
+	hint.ai_protocol = IPPROTO_TCP;
+	hint.ai_flags = AI_PASSIVE;	
+
+	if(getaddrinfo(name, cp, &hint, &list) || !list)
+	{
+		endSocket();
+		error(errBindingFailed, "Could not find service", errno);
+		return;
+	}
+
+#if defined(SO_REUSEADDR) && !defined(WIN32)
+        int opt = 1;
+        setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, 
+                (socklen_t)sizeof(opt));
+#endif
+
+	first = list;
+	while(list)
+	{
+		if(!bind(so, list->ai_addr, (socklen_t)list->ai_addrlen))
+		{
+			state = BOUND;
+			break;
+		}
+		list = list->ai_next;
+	}
+	freeaddrinfo(first);
+	if(state != BOUND)
+	{
+		endSocket();
+                error(errBindingFailed,"Could not bind socket",socket_errno);
+                return;
+	}
+
+	setSegmentSize(mss);
+
+        if(listen(so, backlog))
+        {
+                endSocket();
+                error(errBindingFailed,"Could not listen on socket",socket_errno);
+                return;
+        }
+}
+
+#else
+TCPV6Socket::TCPV6Socket(const char *name, unsigned backlog, unsigned mss) :
+Socket(AF_INET6, SOCK_STREAM, 0)
+{
+	char namebuf[128], *cp;
+        struct sockaddr_in6 addr;
+	struct servent *svc;
+
+	memset(&addr, 0, sizeof(addr));
+        snprintf(namebuf, sizeof(namebuf), "%s", name);
+        cp = strrchr(namebuf, '/');
+
+        if(!cp)
+        {
+                cp = namebuf;
+                name = "*";
+        }
+        else
+        {
+                name = namebuf;
+                *(cp++) = 0;
+        }
+
+	addr.sin6_family = AF_INET6;
+	if(isdigit(*cp))
+		addr.sin6_port = htons(atoi(cp));
+	else
+	{
+		mutex.enter();
+		svc = getservbyname(cp, "tcp");
+		if(svc)
+			addr.sin6_port = svc->s_port;
+		mutex.leave();
+		if(!svc)
+		{
+                	endSocket();
+                	error(errBindingFailed, "Could not find service", errno);
+                	return;
+
+		}
+	}
+
+	IPV6Address ia(name);
+	addr.sin6_addr = getaddress(ia);
+
+#if defined(SO_REUSEADDR) && !defined(WIN32)
+        int opt = 1;    
+        setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, 
+		(socklen_t)sizeof(opt));
+#endif      
+
+        if(bind(so, (struct sockaddr *)&addr, sizeof(addr)))
+        {
+                endSocket();
+                error(errBindingFailed,"Could not bind socket",socket_errno);
+                return;
+        }
+
+	setSegmentSize(mss);
+
+        if(listen(so, backlog)) 
+        {
+                endSocket();
+                error(errBindingFailed,"Could not listen on socket",
+			socket_errno);
+                return;
+        }
+        state = BOUND;
+}
+
+#endif	
+
+TCPV6Socket::TCPV6Socket(const IPV6Address &ia, tpport_t port, unsigned backlog, unsigned mss) :
+Socket(AF_INET6, SOCK_STREAM, 0)
 {
 	struct sockaddr_in6 addr;
 
@@ -1441,6 +2261,8 @@ Socket(AF_INET, SOCK_STREAM, 0)
 		return;
 	}
 
+	setSegmentSize(mss);
+
 	if(listen(so, backlog))
 	{
 		endSocket();
@@ -1454,7 +2276,7 @@ Socket(AF_INET, SOCK_STREAM, 0)
 
 void TCPSocket::reject(void)
 {
-	int rej = accept(so, NULL, NULL);
+	SOCKET rej = accept(so, NULL, NULL);
 	shutdown(rej, 2);
 #ifdef WIN32
 	closesocket(rej);
@@ -1476,8 +2298,8 @@ void TCPV6Socket::reject(void)
 }
 #endif
 
-TCPStream::TCPStream(TCPSocket &server, int size, bool throwflag, timeout_t to) :
-	streambuf(), Socket(accept(server.so, NULL, NULL)),
+TCPStream::TCPStream(TCPSocket &server, bool throwflag, timeout_t to) :
+	streambuf(), Socket(accept(server.getSocket(), NULL, NULL)),
 #ifdef	HAVE_OLD_IOSTREAM
 	iostream()
 #else
@@ -1505,13 +2327,13 @@ TCPStream::TCPStream(TCPSocket &server, int size, bool throwflag, timeout_t to)
 		return;
 	}
 
-	allocate(size);
+	segmentBuffering(server.getSegmentSize());
 	Socket::state = CONNECTED;
 }
 
 #ifdef  CCXX_IPV6
-TCPStream::TCPStream(TCPV6Socket &server, int size, bool throwflag, timeout_t to) :
-	streambuf(), Socket(accept(server.so, NULL, NULL)),
+TCPStream::TCPStream(TCPV6Socket &server, bool throwflag, timeout_t to) :
+	streambuf(), Socket(accept(server.getSocket(), NULL, NULL)),
 #ifdef	HAVE_OLD_IOSTREAM
 	iostream()
 #else
@@ -1540,12 +2362,12 @@ TCPStream::TCPStream(TCPV6Socket &server, int size, bool throwflag, timeout_t to
 		return;
 	}
 
-	allocate(size);
+	segmentBuffering(server.getSegmentSize());
 	Socket::state = CONNECTED;
 }
 #endif
 
-TCPStream::TCPStream(const IPV4Host &host, tpport_t port, int size, bool throwflag, timeout_t to) :
+TCPStream::TCPStream(const IPV4Host &host, tpport_t port, unsigned size, bool throwflag, timeout_t to) :
 	streambuf(), Socket(AF_INET, SOCK_STREAM, 0),
 #ifdef	HAVE_OLD_IOSTREAM
 	iostream(),
@@ -1564,7 +2386,7 @@ TCPStream::TCPStream(const IPV4Host &host, tpport_t port, int size, bool throwfl
 }
 
 #ifdef  CCXX_IPV6
-TCPStream::TCPStream(const IPV6Host &host, tpport_t port, int size, bool throwflag, timeout_t to) :
+TCPStream::TCPStream(const IPV6Host &host, tpport_t port, unsigned size, bool throwflag, timeout_t to) :
 	streambuf(), Socket(AF_INET6, SOCK_STREAM, 0),
 #ifdef	HAVE_OLD_IOSTREAM
 	iostream(),
@@ -1584,7 +2406,131 @@ TCPStream::TCPStream(const IPV6Host &host, tpport_t port, int size, bool throwfl
 }
 #endif
 
-void TCPStream::connect(const IPV4Host &host, tpport_t port, int size)
+#ifdef	HAVE_GETADDRINFO
+
+void TCPStream::connect(const char *target, unsigned mss)
+{
+	char namebuf[128];
+	char *cp;	
+	struct addrinfo hint, *list = NULL, *next, *first;
+	bool connected = false;
+	socklen_t alen = sizeof(mss);
+
+	snprintf(namebuf, sizeof(namebuf), "%s", target);
+	cp = strrchr(namebuf, '/');
+	if(!cp)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp)
+	{
+		endStream();
+		connectError();
+		return;
+	}
+
+	*(cp++) = 0;
+
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = family;
+	hint.ai_socktype = SOCK_STREAM;
+	hint.ai_protocol = IPPROTO_TCP;
+
+	if(getaddrinfo(namebuf, cp, &hint, &list) || !list)
+	{
+		endStream();
+		connectError();
+		return;
+	}
+
+	first = list;
+
+#ifdef	TCP_MAXSEG
+	if(mss)
+		setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, sizeof(mss));
+#endif
+	
+	while(list)
+	{
+		if(!::connect(so, list->ai_addr, (socklen_t)list->ai_addrlen))
+		{
+			connected = true;
+			break;
+		}
+		next = list->ai_next;
+		list = next;
+	}
+
+	freeaddrinfo(first);
+
+	if(!connected)
+	{
+		endStream();
+		connectError();
+		return;
+	}
+	
+	segmentBuffering(mss);
+	Socket::state = CONNECTED;
+}
+
+#else
+
+void TCPStream::connect(const char *target, unsigned mss)
+{
+        char namebuf[128];
+        char *cp;
+        bool connected = false;
+	struct servent *svc;
+	tpport_t port;
+
+        snprintf(namebuf, sizeof(namebuf), "%s", target);
+        cp = strrchr(namebuf, '/');
+        if(!cp)
+                cp = strrchr(namebuf, ':');
+
+        if(!cp)
+        {
+                endStream();
+                connectError();
+                return;
+        }
+
+	if(isdigit(*cp))
+		port = atoi(cp);
+	else
+	{
+		mutex.enter();
+		svc = getservbyname(cp, "tcp");
+		if(svc)
+			port = ntohs(svc->s_port);
+		mutex.leave();
+		if(!svc)
+		{
+			endStream();
+			connectError();
+			return;
+		}
+	}
+	
+	switch(family)
+	{
+	case IPV4:
+		connect(IPV4Host(namebuf), port, mss);
+		break;
+#ifdef	CCXX_IPV6
+	case IPV6:
+		connect(IPV6Host(namebuf), port, mss);
+		break;
+#endif
+	default:
+		endStream();
+		connectError();
+	}
+}
+
+#endif
+
+void TCPStream::connect(const IPV4Host &host, tpport_t port, unsigned mss)
 {
 	size_t i;
 	fd_set fds;
@@ -1593,6 +2539,12 @@ void TCPStream::connect(const IPV4Host &host, tpport_t port, int size)
 	int rtn;
 	long sockopt;
 	socklen_t len = sizeof(sockopt);
+	socklen_t alen = sizeof(mss);
+
+#ifdef	TCP_MAXSEG
+	if(mss)
+		setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, sizeof(mss));
+#endif
 
 	for(i = 0 ; i < host.getAddressCount(); i++)
 	{
@@ -1628,7 +2580,7 @@ void TCPStream::connect(const IPV4Host &host, tpport_t port, int size)
 			
 			// timeout check for connect completion
 
-			if(::select(so + (SOCKET)1, NULL, &fds, NULL, &to) < 1)
+			if(::select((int)so + 1, NULL, &fds, NULL, &to) < 1)
 				continue;
 
 			getsockopt(so, SOL_SOCKET, SO_ERROR, (char *)&sockopt, &len);
@@ -1654,12 +2606,12 @@ void TCPStream::connect(const IPV4Host &host, tpport_t port, int size)
 		return;
 	}
 
-	allocate(size);
+	segmentBuffering(mss);
 	Socket::state = CONNECTED;
 }
 
 #ifdef  CCXX_IPV6
-void TCPStream::connect(const IPV6Host &host, tpport_t port, int size)
+void TCPStream::connect(const IPV6Host &host, tpport_t port, unsigned mss)
 {
 	size_t i;
 	fd_set fds;
@@ -1668,6 +2620,12 @@ void TCPStream::connect(const IPV6Host &host, tpport_t port, int size)
 	int rtn;
 	long sockopt;
 	socklen_t len = sizeof(sockopt);
+	socklen_t alen = sizeof(mss);
+
+#ifdef	TCP_MAXSEG
+	if(mss)
+		setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, sizeof(mss));
+#endif
 
 	for(i = 0 ; i < host.getAddressCount(); i++)
 	{
@@ -1729,19 +2687,37 @@ void TCPStream::connect(const IPV6Host &host, tpport_t port, int size)
 		return;
 	}
 
-	allocate(size);
+	segmentBuffering(mss);
 	Socket::state = CONNECTED;
 }
 #endif
 
-TCPStream::TCPStream(Family fam, bool throwflag) :
+TCPStream::TCPStream(const char *target, Family fam, unsigned mss, bool throwflag, timeout_t to) :
+        streambuf(), Socket(PF_INET, SOCK_STREAM, 0),
+#ifdef  HAVE_OLD_IOSTREAM
+        iostream(),
+#else
+        iostream((streambuf *)this),
+#endif
+	timeout(to),
+	bufsize(0),gbuf(NULL),pbuf(NULL)
+{
+	family = fam;
+#ifdef	HAVE_OLD_IOSTREAM
+	init((streambuf *)this);
+#endif
+	setError(throwflag);
+	connect(target, mss);
+}
+
+TCPStream::TCPStream(Family fam, bool throwflag, timeout_t to) :
 	streambuf(), Socket(PF_INET, SOCK_STREAM, 0),
 #ifdef	HAVE_OLD_IOSTREAM
 	iostream(),
 #else
         iostream((streambuf *)this),
 #endif
-	timeout(0),
+	timeout(to),
         bufsize(0),gbuf(NULL),pbuf(NULL)
 {
 	family = fam;
@@ -1767,6 +2743,128 @@ iostream((streambuf *)this)
 	allocate(bufsize);
 }
 
+void TCPStream::connect(TCPSocket &tcpip)
+{
+	tpport_t port;
+	
+	endStream();
+	family = IPV4;
+	so = accept(tcpip.getSocket(), NULL, NULL);
+	if(so == INVALID_SOCKET)
+		return;
+
+	IPV4Host host = getPeer(&port);
+	if(!tcpip.onAccept(host, port))
+	{
+		endSocket();
+		clear(ios::failbit | rdstate());
+		return;
+	}
+
+	segmentBuffering(tcpip.getSegmentSize());
+	Socket::state = CONNECTED;
+}	
+
+#ifdef	CCXX_IPV6
+
+void TCPStream::connect(TCPV6Socket &tcpip)
+{
+        tpport_t port;
+
+        endStream();
+        family = IPV6;
+        so = accept(tcpip.getSocket(), NULL, NULL);
+        if(so == INVALID_SOCKET)
+                return;
+
+        IPV6Host host = getIPV6Peer(&port);
+        if(!tcpip.onAccept(host, port))
+        {
+                endSocket();
+                clear(ios::failbit | rdstate());
+                return;
+        }
+
+	segmentBuffering(tcpip.getSegmentSize());
+        Socket::state = CONNECTED;
+}
+#endif
+
+void TCPStream::segmentBuffering(unsigned mss)
+{
+	unsigned max = 0;
+	socklen_t alen = sizeof(max);
+
+	if(mss == 1)	// special interactive
+	{
+		allocate(1);
+		return;
+	}
+
+#ifdef	TCP_MAXSEG
+	if(mss)
+		setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&max, sizeof(max));
+	getsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&max, &alen);
+#endif
+
+	if(max && max < mss)
+		mss = max;
+
+	if(!mss)
+	{
+		if(max)
+			mss = max;
+		else
+			mss = 536;
+		allocate(mss);
+		return;
+	}
+
+#ifdef	TCP_MAXSEG
+	setsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, sizeof(mss));
+#endif
+
+	if(mss < 80)
+		mss = 80;
+
+	if(mss * 7 < 64000)
+		bufferSize(mss * 7);
+	else if(mss * 6 < 64000)
+		bufferSize(mss * 6);
+	else
+		bufferSize(mss * 5);
+
+	if(mss < 512)
+		sendLimit(mss * 4);
+
+	allocate(mss);
+}	
+
+int TCPStream::getSegmentSize(void)
+{
+	unsigned mss = 0;
+#ifdef	TCP_MAXSEG
+	socklen_t alen = sizeof(mss);
+	
+	getsockopt(so, IPPROTO_TCP, TCP_MAXSEG, (char *)&mss, &alen);
+#endif
+	if(!mss)
+		return (int)bufsize;
+
+	return mss;	
+}
+
+void TCPStream::disconnect(void)
+{
+	if(Socket::state == AVAILABLE)
+		return;
+
+	endStream();
+	so = socket(family, SOCK_STREAM, 0);
+	if(so != INVALID_SOCKET)
+		Socket::state = AVAILABLE;
+}
+
 void TCPStream::endStream(void)
 {
 	if(bufsize) sync();
@@ -1780,7 +2878,7 @@ void TCPStream::endStream(void)
 	endSocket();
 }
 
-void TCPStream::allocate(int size)
+void TCPStream::allocate(size_t size)
 {
 	if(size < 2)
 	{
@@ -1865,7 +2963,7 @@ int TCPStream::underflow()
 
 	rlen = (ssize_t)((gbuf + bufsize) - eback());
 	if(Socket::state == STREAM)
-		rlen = ::read((int)so, (char *)eback(), IOLEN rlen);
+		rlen = ::read((int)so, (char *)eback(), _IOLEN64 rlen);
 	else if(timeout && !Socket::isPending(pendingInput, timeout))
 	{
 		clear(ios::failbit | rdstate());
@@ -1910,7 +3008,7 @@ int TCPStream::sync(void)
 }
 
 #ifdef	HAVE_SNPRINTF
-int TCPStream::printf(const char *format, ...)
+size_t TCPStream::printf(const char *format, ...)
 {
 	va_list args;
 	size_t len;
@@ -1924,9 +3022,9 @@ int TCPStream::printf(const char *format, ...)
 	va_end(args);
 	len = strlen(buf);
 	if(Socket::state == STREAM)
-		return ::write((int)so, buf, IOLEN len);
+		return ::write((int)so, buf, _IOLEN64 len);
 	else
-		return ::send(so, buf, IOLEN len, 0);
+		return ::send(so, buf, _IOLEN64 len, 0);
 }
 #endif
 
@@ -1996,88 +3094,8 @@ int TCPStream::overflow(int c)
 	return c;
 }
 
-tcpstream::tcpstream() :
-TCPStream()
-{
-	setError(false);	/* no exceptions */
-}
-
-tcpstream::tcpstream(const char *addr, int buf) :
-TCPStream()
-{
-	setError(false);
-	open(addr, buf);
-}
-
-tcpstream::tcpstream(TCPSocket &server, int buf) :
-TCPStream()
-{
-	setError(false);
-	open(server, buf);
-}
-
-bool tcpstream::operator!() const
-{
-	return (Socket::state != CONNECTED) ? true : false;
-}
-
-void tcpstream::open(TCPSocket &tcpip, int buf)
-{
-	tpport_t port;
-
-	endStream();
-	so = accept(tcpip.so, NULL, NULL);
-	if(so == INVALID_SOCKET)
-		return;
-
-	IPV4Host host = getPeer(&port);
-	if(!tcpip.onAccept(host, port))
-	{
-		endSocket();
-		clear(ios::failbit | rdstate());
-		return;
-	}
-
-	allocate(buf);
-	Socket::state = CONNECTED;
-}
-	
-void tcpstream::open(const char *path, int buf)
-{
-	char abuf[256];
-	const char *cp;
-	char *ep;
-
-	close();
-	if(Socket::state != AVAILABLE)
-		return;
-	
-	cp = strrchr(path, ':');
-	if(!cp)
-		return;
-
-	++cp;
-	setString(abuf, sizeof(abuf), path);
-	ep = strrchr(abuf, ':');
-	if(ep)
-		*ep = 0;
-
-	connect(IPV4Host(abuf), (tpport_t)atoi(cp), buf);
-}
-	
-void tcpstream::close(void)
-{
-	if(Socket::state == AVAILABLE)
-		return;
-
-	endStream();
-	so = socket(AF_INET, SOCK_STREAM, 0);
-	if(so != INVALID_SOCKET)
-		Socket::state = AVAILABLE;
-}
-
 TCPSession::TCPSession(const IPV4Host &ia, 
-		       tpport_t port, int size, int pri, int stack) :
+		       tpport_t port, size_t size, int pri, size_t stack) :
 TCPStream(IPV4), Thread(pri, stack)
 {
 	setCompletion(false);
@@ -2124,7 +3142,7 @@ TCPStream(IPV4), Thread(pri, stack)
 
 #ifdef  CCXX_IPV6
 TCPSession::TCPSession(const IPV6Host &ia, 
-		       tpport_t port, int size, int pri, int stack) :
+		       tpport_t port, size_t size, int pri, size_t stack) :
 TCPStream(IPV6), Thread(pri, stack)
 {
 	setCompletion(false);
@@ -2171,18 +3189,16 @@ TCPStream(IPV6), Thread(pri, stack)
 #endif
 
 
-TCPSession::TCPSession(TCPSocket &s,
-		       int size, int pri, int stack) :
-TCPStream(s, size), Thread(pri, stack)
+TCPSession::TCPSession(TCPSocket &s, int pri, size_t stack) :
+TCPStream(s), Thread(pri, stack)
 {
 	setCompletion(true);
 	setError(false);
 }
 
 #ifdef  CCXX_IPV6
-TCPSession::TCPSession(TCPV6Socket &s,
-		       int size, int pri, int stack) :
-TCPStream(s, size), Thread(pri, stack)
+TCPSession::TCPSession(TCPV6Socket &s, int pri, size_t stack) :
+TCPStream(s), Thread(pri, stack)
 {
 	setCompletion(true);
 	setError(false);
