@@ -72,7 +72,7 @@ const unsigned String::minsize = ((sizeof(char *) + (sizeof(unsigned) * 2) + 1))
 const unsigned String::slotsize = 32;
 const unsigned String::pagesize = 1024;
 const unsigned String::slotlimit = 512;
-const unsigned String::slots = ((slotlimit / slotsize) + 1);
+const unsigned String::slotcount = ((slotlimit / slotsize) + 1);
 const size_t String::npos = (size_t)(-1);
 
 String::String()
@@ -662,8 +662,8 @@ char *String::getSpace(size_t chars)
 	if(!pager)
 	{
 		pager = new MemPager(pagesize);
-		idx = (char **)pager->alloc(sizeof(char *) * slots);
-		memset(idx, 0, sizeof(char *) * slots);
+		idx = (char **)pager->alloc(sizeof(char *) * slotcount);
+		memset(idx, 0, sizeof(char *) * slotcount);
 	}
 	cp = idx[slot];
 	if(cp)
@@ -778,7 +778,7 @@ const size_t String::getLength(void) const
 	return content.ministring.length;
 }
 
-String operator+(const String &s1, char c2)
+String operator+(const String &s1, const char c2)
 {
 	String result(s1);
 	result.add(c2);
@@ -806,7 +806,7 @@ String operator+(const char *s1, const String &s2)
 	return result;
 }
 
-String operator+(char c1, const String &s2)
+String operator+(const char c1, const String &s2)
 {
 	String result(1, c1);
 	result += s2;
@@ -925,6 +925,45 @@ std::ostream &operator<<(std::ostream &os, const String &str)
 	return os;
 }
 
+void *StringObject::operator new(size_t size) NEW_THROWS
+{
+	char *base;
+	size_t *sp;
+	size += sizeof(size_t);
+
+	if(size > String::slotlimit)
+		return NULL;
+
+	base = String::getSpace(size);
+
+	if(!base)
+		return NULL;
+
+	sp = (size_t *)base;
+	*sp = size;
+	base += sizeof(size_t);
+	return (void *)base;
+}	
+
+void StringObject::operator delete(void *ptr)
+{
+	char **next;
+	unsigned slot;
+	size_t *size = (size_t *)ptr;
+	
+	--size;
+	ptr = size;
+
+	slot = (unsigned)(((*size) - 1) / String::slotsize);
+	next = ((char **)(ptr));
+	mutex.enter();
+	*next = String::idx[slot];
+	String::idx[slot] = (char *)ptr;
+	mutex.leave();
+}
+	
+	
+
 std::istream &getline(std::istream &is, String &str, char delim, size_t len)
 {
 	if(!len)
@@ -934,7 +973,7 @@ std::istream &getline(std::istream &is, String &str, char delim, size_t len)
 		str.resize(len + 1);
 
 	char *ptr = str.getText();
-	is.getline(ptr, len, delim);
+	is.getline(ptr, (std::streamsize)len, delim);
 	str.setLength(strlen(ptr));
 	return is;
 }
