@@ -409,7 +409,7 @@ class MainThread : public Thread
 protected:
 	void run(void) {return;};
 #ifndef WIN32
-	void onSignal(int signo) { ::exit(signo);};
+	void onSignal(int signo) { std::exit(signo);};
 #endif
 
 public:
