@@ -175,14 +175,13 @@ void URLStream::setProxy(const char *host, tpport_t port)
 		v6proxyHost = host;
 		break;
 #endif
-	default:
+	case IPV4:
 		proxyHost = host;
 	}
 	proxyPort = port;
 }
 
-URLStream::Error URLStream::submit(const char *path, const char **vars,
-	       int buf)
+URLStream::Error URLStream::submit(const char *path, const char **vars, size_t buf)
 {
 	Error status = errInvalid, saved;
 
@@ -213,8 +212,7 @@ URLStream::Error URLStream::submit(const char *path, const char **vars,
 	}
 }
 
-URLStream::Error URLStream::post(const char *path, const char **vars,
-		int buf)
+URLStream::Error URLStream::post(const char *path, const char **vars, size_t buf)
 {
 	Error status = errInvalid, saved;
 
@@ -247,7 +245,7 @@ URLStream::Error URLStream::post(const char *path, const char **vars,
 }
 
 
-URLStream::Error URLStream::head(const char *path, int buf)
+URLStream::Error URLStream::head(const char *path, size_t buf)
 {
 	Error status = errInvalid, saved;
 
@@ -299,7 +297,7 @@ URLStream &URLStream::getline(char *buffer, size_t size)
 	return *this;
 }
 
-URLStream::Error URLStream::get(int buffer)
+URLStream::Error URLStream::get(size_t buffer)
 {
 	String path = String("http://") + m_host;
 
@@ -311,7 +309,7 @@ URLStream::Error URLStream::get(int buffer)
 	return get(path.c_str(), buffer);
 }
 
-URLStream::Error URLStream::get(const char *urlpath, int buf)
+URLStream::Error URLStream::get(const char *urlpath, size_t buf)
 {
 	const char *path = urlpath;
 	Error status = errInvalid, saved;
@@ -439,8 +437,7 @@ void URLStream::close(void)
 		Socket::state = AVAILABLE;
 }
 
-URLStream::Error URLStream::sendHTTPHeader(const char *url, const char **vars,
-		int buf)
+URLStream::Error URLStream::sendHTTPHeader(const char *url, const char **vars, size_t buf)
 {
 	// TODO: implement authentication
 	char reloc[4096];
@@ -461,6 +458,9 @@ URLStream::Error URLStream::sendHTTPHeader(const char *url, const char **vars,
 	tpport_t port = 80;
 	const char **args = vars;
 	const char *var;
+	bool lasteq = true;
+
+	struct servent *svc;
 
 retry:
 #ifdef HAVE_SSTREAM
@@ -478,7 +478,17 @@ retry:
 	ep = strrchr(hp, ':');
 	if(ep)
 	{
-		port = atoi(ep+1);
+		++ep;
+		if(isdigit(*ep))
+			port = atoi(ep);
+		else
+		{
+			Socket::mutex.enter();
+			svc = getservbyname(ep, "tcp");
+			if(svc)
+				port = ntohs(svc->s_port);
+			Socket::mutex.leave();
+		}
 		*ep = 0;
 	}
 
@@ -523,9 +533,18 @@ retry:
 		str << "?";
 		while(*vars)
 		{
-			if(count++)
+			if(count++ && lasteq)
 				str << "&";
 			str << *vars;
+			if(!lasteq)
+				lasteq = true;
+			else if(strchr(*vars, '='))
+				lasteq = true;
+			else
+			{
+				lasteq = false;
+				str << "=";
+			}				
 			++vars;
 		}
 	}
@@ -595,7 +614,7 @@ retry:
 		while(*args)
 		{
 			var = *args;
-			if(count++)
+			if(count++ || !strchr(var, '='))
 				len += strlen(var) + 1;
 			else
 				len = strlen(var);
@@ -677,11 +696,11 @@ retry:
 		{
 #ifdef	CCXX_IPV6
 		case IPV6:
-			connect(v6proxyHost, proxyPort, buf);
+			connect(v6proxyHost, proxyPort, (unsigned)buf);
 			break;
 #endif
 		default:	
-		        connect(proxyHost, proxyPort, buf);
+		        connect(proxyHost, proxyPort, (unsigned)buf);
 		}
 	}
 	else
@@ -704,9 +723,9 @@ retry:
 	// FIXME: send (or write) can send less than len bytes
 	// use stream funcion ??
 #ifdef HAVE_SSTREAM
-	::send(so, str.str().c_str(), IOLEN str.str().length(), 0);
+	::send(so, str.str().c_str(), _IOLEN64 str.str().length(), 0);
 #else
-	::send(so, buffer, IOLEN strlen(buffer), 0);
+	::send(so, buffer, _IOLEN64 strlen(buffer), 0);
 #endif
 
 	if(urlmethod == methodHttpPost && vars)
@@ -724,7 +743,13 @@ retry:
                         else
                                 sep = true;
 			var = *vars;
-			::send(so, var, IOLEN strlen(var),0);
+			if(!strchr(var, '='))
+			{
+				snprintf(reloc, sizeof(reloc), "%s=%s", var, *(++vars));
+				::send(so, reloc, _IOLEN64 strlen(reloc), 0);
+			}
+			else
+				::send(so, var, _IOLEN64 strlen(var),0);
 			++vars;
 		}
 		::send(so, "\r\n", 2, 0);
@@ -818,6 +843,7 @@ void URLStream::setProxyAuthentication(Authentication a, const char *value)
  	{
  		if(!proxyUser)
  			proxyUser = "anonymous";
+
  		if(!proxyPasswd)
  			proxyPasswd = "";
 	}
