@@ -70,7 +70,7 @@ using namespace std;
 static const unsigned char alphabet[65] = 
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
-CCXX_EXPORT(char*) b64Encode(const char *source, char *dest, unsigned limit)
+CCXX_EXPORT(char*) b64Encode(const char *source, char *dest, size_t limit)
 {
 	b64Encode((const unsigned char*)source,strlen(source),
 			dest,limit);
@@ -228,10 +228,10 @@ CCXX_EXPORT(char*) urlDecode(char *source, char *dest)
 	return ret;
 }	
 
-CCXX_EXPORT(char*) urlEncode(const char *source, char *dest, unsigned max)	
+CCXX_EXPORT(char*) urlEncode(const char *source, char *dest, size_t max)	
 {
 	static const char *hex = "0123456789abcdef";
-	unsigned len = 0;
+	size_t len = 0;
 	unsigned char ch;
 	char *ret = dest;
 
